@@ -13,8 +13,11 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import urllib
 from collections import defaultdict
 
+import datetime
+
 import github3.exceptions
 import re
 import time
@@ -24,25 +27,16 @@ from requests import HTTPError
 from requests.structures import CaseInsensitiveDict
 
 from tests.fake_graphql import FakeGithubQuery
+from zuul.driver.github.githubconnection import utc
 
 FAKE_BASE_URL = 'https://example.com/api/v3/'
 
 
-class ErrorResponse:
-    status_code = 0
-    message = ''
-
-    def json(self):
-        return {
-            'message': self.message
-        }
-
-
 class FakeUser(object):
     def __init__(self, login):
         self.login = login
-        self.name = "Github User"
-        self.email = "github.user@example.com"
+        self.name = login
+        self.email = '%s@example.com' % login
         self.html_url = 'https://example.com/%s' % login
 
 
@@ -62,13 +56,18 @@ class FakeBranch(object):
         }
 
 
+class FakeCreator:
+    def __init__(self, login):
+        self.login = login
+
+
 class FakeStatus(object):
     def __init__(self, state, url, description, context, user):
         self.state = state
         self.context = context
+        self.creator = FakeCreator(user)
         self._url = url
         self._description = description
-        self._user = user
 
     def as_dict(self):
         return {
@@ -77,20 +76,32 @@ class FakeStatus(object):
             'description': self._description,
             'context': self.context,
             'creator': {
-                'login': self._user
+                'login': self.creator.login
             }
         }
 
 
+class FakeApp:
+    def __init__(self, name, slug):
+        self.name = name
+        self.slug = slug
+
+
 class FakeCheckRun(object):
-    def __init__(self, name, details_url, output, status, conclusion,
-                 completed_at, app):
+    def __init__(self, id, name, details_url, output, status, conclusion,
+                 completed_at, external_id, actions, app):
+        if actions is None:
+            actions = []
+
+        self.id = id
         self.name = name
         self.details_url = details_url
         self.output = output
         self.conclusion = conclusion
         self.completed_at = completed_at
-        self.app = app
+        self.external_id = external_id
+        self.actions = actions
+        self.app = FakeApp(name=app, slug=app)
 
         # Github automatically sets the status to "completed" if a conclusion
         # is provided.
@@ -101,22 +112,29 @@ class FakeCheckRun(object):
 
     def as_dict(self):
         return {
+            'id': self.id,
             "name": self.name,
             "status": self.status,
             "output": self.output,
             "details_url": self.details_url,
             "conclusion": self.conclusion,
             "completed_at": self.completed_at,
+            "external_id": self.external_id,
+            "actions": self.actions,
             "app": {
-                "slug": self.app,
+                "slug": self.app.slug,
+                "name": self.app.name,
             },
         }
 
-    def update(self, conclusion, completed_at, output, details_url):
+    def update(self, conclusion, completed_at, output, details_url,
+               external_id, actions):
         self.conclusion = conclusion
         self.completed_at = completed_at
         self.output = output
         self.details_url = details_url
+        self.external_id = external_id
+        self.actions = actions
 
         # As we are only calling the update method when a build is completed,
         # we can always set the status to "completed".
@@ -151,14 +169,24 @@ class FakeCommit(object):
         # the last status provided for a commit.
         self._statuses.insert(0, status)
 
-    def set_check_run(self, name, details_url, output, status, conclusion,
-                      completed_at, app):
+    def set_check_run(self, id, name, details_url, output, status, conclusion,
+                      completed_at, external_id, actions, app):
         check_run = FakeCheckRun(
-            name, details_url, output, status, conclusion, completed_at, app
+            id,
+            name,
+            details_url,
+            output,
+            status,
+            conclusion,
+            completed_at,
+            external_id,
+            actions,
+            app,
         )
         # Always insert a check_run to the front of the list to represent the
         # last check_run provided for a commit.
         self._check_runs.insert(0, check_run)
+        return check_run
 
     def get_url(self, path, params=None):
         if path == 'statuses':
@@ -196,6 +224,7 @@ class FakeRepository(object):
         self._commits = {}
         self.data = data
         self.name = name
+        self.check_run_counter = 0
 
         # Simple dictionary to store permission values per feature (e.g.
         # checks, Repository contents, Pull requests, Commit statuses, ...).
@@ -219,14 +248,16 @@ class FakeRepository(object):
         return self._branches
 
     def _set_branch_protection(self, branch_name, protected=True,
-                               contexts=None):
-        if not protected and branch_name in self._branch_protection_rules:
-            del self._branch_protection_rules[branch_name]
+                               contexts=None, require_review=False):
+        if not protected:
+            if branch_name in self._branch_protection_rules:
+                del self._branch_protection_rules[branch_name]
             return
 
         rule = self._branch_protection_rules[branch_name]
         rule.pattern = branch_name
         rule.required_contexts = contexts or []
+        rule.require_reviews = require_review
 
     def _set_permission(self, key, value):
         # NOTE (felix): Currently, this is only used to mock a repo with
@@ -242,7 +273,7 @@ class FakeRepository(object):
         return FAKE_BASE_URL + fakepath
 
     def _get(self, url, headers=None):
-        client = FakeGithubClient(self.data)
+        client = FakeGithubClient(data=self.data)
         return client.session.get(url, headers)
 
     def _create_branch(self, branch):
@@ -263,7 +294,7 @@ class FakeRepository(object):
 
     def create_check_run(self, head_sha, name, details_url=None, output=None,
                          status=None, conclusion=None, completed_at=None,
-                         app="zuul"):
+                         external_id=None, actions=None, app="zuul"):
 
         # Raise the appropriate github3 exception in case we don't have
         # permission to access the checks API
@@ -278,18 +309,25 @@ class FakeRepository(object):
         if commit is None:
             commit = FakeCommit(head_sha)
             self._commits[head_sha] = commit
+        self.check_run_counter += 1
         commit.set_check_run(
-            name, details_url, output, status, conclusion, completed_at, app)
+            str(self.check_run_counter),
+            name,
+            details_url,
+            output,
+            status,
+            conclusion,
+            completed_at,
+            external_id,
+            actions,
+            app,
+        )
 
     def commit(self, sha):
 
         if self.fail_not_found > 0:
             self.fail_not_found -= 1
-
-            resp = ErrorResponse()
-            resp.status_code = 404
-            resp.message = 'Not Found'
-
+            resp = FakeResponse(404, 'Not found')
             raise github3.exceptions.NotFoundError(resp)
 
         commit = self._commits.get(sha, None)
@@ -464,7 +502,8 @@ class FakePull(object):
 
     @property
     def head(self):
-        client = FakeGithubClient(self._fake_pull_request.github.github_data)
+        client = FakeGithubClient(
+            data=self._fake_pull_request.github.github_data)
         repo = client.repo_from_project(self._fake_pull_request.project)
         return repo.commit(self._fake_pull_request.head_sha)
 
@@ -473,23 +512,6 @@ class FakePull(object):
         # with the head_sha as the only commit
         return [self.head]
 
-    def merge(self, commit_message=None, sha=None, merge_method=None):
-        conn = self._fake_pull_request.github
-        pr = self._fake_pull_request
-
-        # record that this got reported
-        conn.reports.append((pr.project, pr.number, 'merge', merge_method))
-        if conn.merge_failure:
-            raise Exception('Unknown merge failure')
-        if conn.merge_not_allowed_count > 0:
-            conn.merge_not_allowed_count -= 1
-            resp = ErrorResponse()
-            resp.status_code = 403
-            resp.message = 'Merge not allowed'
-            raise github3.exceptions.MethodNotAllowed(resp)
-        pr.setMerged(commit_message)
-        return True
-
     def as_dict(self):
         pr = self._fake_pull_request
         connection = pr.github
@@ -509,6 +531,9 @@ class FakePull(object):
                 },
                 'ref': pr.branch,
             },
+            'user': {
+                'login': 'octocat'
+            },
             'draft': pr.draft,
             'mergeable': True,
             'state': pr.state,
@@ -521,6 +546,7 @@ class FakePull(object):
             },
             'merged': pr.is_merged,
             'body': pr.body,
+            'body_text': pr.body_text,
             'changed_files': len(pr.files),
             'labels': [{'name': l} for l in pr.labels]
         }
@@ -533,8 +559,9 @@ class FakeIssueSearchResult(object):
 
 
 class FakeResponse(object):
-    def __init__(self, data, status_code=200):
+    def __init__(self, data, status_code=200, status_message='OK'):
         self.status_code = status_code
+        self.status_message = status_message
         self.data = data
         self.links = {}
 
@@ -549,8 +576,11 @@ class FakeResponse(object):
 
     def raise_for_status(self):
         if 400 <= self.status_code < 600:
-            raise HTTPError('{} Something went wrong'.format(self.status_code),
-                            response=self)
+            if isinstance(self.data, str):
+                text = '{} {}'.format(self.status_code, self.data)
+            else:
+                text = '{} {}'.format(self.status_code, self.status_message)
+            raise HTTPError(text, response=self)
 
 
 class FakeGithubSession(object):
@@ -561,6 +591,11 @@ class FakeGithubSession(object):
         self._base_url = None
         self.schema = graphene.Schema(query=FakeGithubQuery)
 
+        # Imitate hooks dict. This will be unused and ignored in the tests.
+        self.hooks = {
+            'response': []
+        }
+
     def build_url(self, *args):
         fakepath = '/'.join(args)
         return FAKE_BASE_URL + fakepath
@@ -580,15 +615,129 @@ class FakeGithubSession(object):
 
     def post(self, url, data=None, headers=None, params=None, json=None):
 
+        # Handle graphql
         if json and json.get('query'):
             query = json.get('query')
             variables = json.get('variables')
             result = self.schema.execute(
-                query, variables=variables, context=self.client._data)
+                query, variables=variables, context=self.client)
+            if result.errors:
+                # Note that github really returns 200 and an errors field in
+                # case of an error.
+                return FakeResponse({'errors': result.errors}, 200)
             return FakeResponse({'data': result.data}, 200)
 
+        # Handle creating comments
+        match = re.match(r'.+/repos/(.+)/issues/(\d+)/comments$', url)
+        if match:
+            project, pr_number = match.groups()
+            project = urllib.parse.unquote(project)
+            self.client._data.reports.append((project, pr_number, 'comment'))
+            pull_request = self.client._data.pull_requests[int(pr_number)]
+            pull_request.addComment(json['body'])
+            return FakeResponse(None, 200)
+
+        # Handle access token creation
+        if re.match(r'.*/app/installations/.*/access_tokens', url):
+            expiry = (datetime.datetime.now(utc) + datetime.timedelta(
+                minutes=60)).replace(microsecond=0).isoformat()
+            data = {
+                'token': 'fake',
+                'expires_at': expiry,
+            }
+            return FakeResponse(data, 201)
+
+        # Handle check run creation
+        match = re.match(r'.*/repos/(.*)/check-runs$', url)
+        if match:
+            if self.client._data.fail_check_run_creation:
+                return FakeResponse('Internal server error', 500)
+
+            org, reponame = match.groups()[0].split('/', 1)
+            repo = self.client._data.repos.get((org, reponame))
+
+            if repo._permissions.get("checks") is False:
+                # To create a proper github3 exception, we need to mock a
+                # response object
+                return FakeResponse(
+                    "Resource not accessible by integration", 403)
+
+            head_sha = json.get('head_sha')
+            commit = repo._commits.get(head_sha, None)
+            if commit is None:
+                commit = FakeCommit(head_sha)
+                repo._commits[head_sha] = commit
+                repo.check_run_counter += 1
+            check_run = commit.set_check_run(
+                str(repo.check_run_counter),
+                json['name'],
+                json['details_url'],
+                json['output'],
+                json.get('status'),
+                json.get('conclusion'),
+                json.get('completed_at'),
+                json['external_id'],
+                json['actions'],
+                json.get('app', 'zuul'),
+            )
+
+            return FakeResponse(check_run.as_dict(), 201)
+
         return FakeResponse(None, 404)
 
+    def put(self, url, data=None, headers=None, params=None, json=None):
+        # Handle pull request merge
+        match = re.match(r'.+/repos/(.+)/pulls/(\d+)/merge$', url)
+        if match:
+            project, pr_number = match.groups()
+            project = urllib.parse.unquote(project)
+            pr = self.client._data.pull_requests[int(pr_number)]
+            conn = pr.github
+
+            # record that this got reported
+            self.client._data.reports.append(
+                (pr.project, pr.number, 'merge', json["merge_method"]))
+            if conn.merge_failure:
+                raise Exception('Unknown merge failure')
+            if conn.merge_not_allowed_count > 0:
+                conn.merge_not_allowed_count -= 1
+                # GitHub returns 405 Method not allowed with more details in
+                # the body of the response.
+                data = {
+                    'message': 'Merge not allowed because of fake reason',
+                }
+                return FakeResponse(data, 405, 'Method not allowed')
+            pr.setMerged(json["commit_message"])
+            return FakeResponse({"merged": True}, 200)
+
+        return FakeResponse(None, 404)
+
+    def patch(self, url, data=None, headers=None, params=None, json=None):
+
+        # Handle check run update
+        match = re.match(r'.*/repos/(.*)/check-runs/(.*)$', url)
+        if match:
+            org, reponame = match.groups()[0].split('/', 1)
+            check_run_id = match.groups()[1]
+            repo = self.client._data.repos.get((org, reponame))
+
+            # Find the specified check run
+            check_runs = [
+                check_run
+                for commit in repo._commits.values()
+                for check_run in commit._check_runs
+                if check_run.id == check_run_id
+            ]
+            check_run = check_runs[0]
+
+            check_run.update(json['conclusion'],
+                             json['completed_at'],
+                             json['output'],
+                             json['details_url'],
+                             json['external_id'],
+                             json['actions'])
+            return FakeResponse(check_run.as_dict(), 200)
+
     def get_repo(self, request, params=None):
         org, project, request = request.split('/', 2)
         project_name = '{}/{}'.format(org, project)
@@ -597,6 +746,10 @@ class FakeGithubSession(object):
 
         return repo.get_url(request, params=params)
 
+    def mount(self, prefix, adapter):
+        # Don't care in tests
+        pass
+
 
 class FakeBranchProtectionRule:
 
@@ -611,14 +764,23 @@ class FakeGithubData(object):
     def __init__(self, pull_requests):
         self.pull_requests = pull_requests
         self.repos = {}
+        self.reports = []
+        self.fail_check_run_creation = False
 
 
 class FakeGithubClient(object):
-    def __init__(self, data, inst_id=None):
+
+    def __init__(self, session=None, data=None):
         self._data = data
-        self._inst_id = inst_id
+        self._inst_id = None
         self.session = FakeGithubSession(self)
 
+    def setData(self, data):
+        self._data = data
+
+    def setInstId(self, inst_id):
+        self._inst_id = inst_id
+
     def user(self, login):
         return FakeUser(login)
 
@@ -642,6 +804,13 @@ class FakeGithubClient(object):
 
     def pull_request(self, owner, project, number):
         fake_pr = self._data.pull_requests[int(number)]
+        repo = self.repository(owner, project)
+        # Ensure a commit for the head_sha exists so this can be resolved in
+        # graphql queries.
+        repo._commits.setdefault(
+            fake_pr.head_sha,
+            FakeCommit(fake_pr.head_sha)
+        )
         return FakePull(fake_pr)
 
     def search_issues(self, query):
@@ -658,9 +827,24 @@ class FakeGithubClient(object):
             return re.match(r'[a-z0-9]{40}', s)
 
         if query_is_sha(query):
-            return (FakeIssueSearchResult(FakeIssue(pr))
-                    for pr in self._data.pull_requests.values()
-                    if pr.head_sha == query)
+            # Github returns all PR's that contain the sha in their history
+            result = []
+            for pr in self._data.pull_requests.values():
+                # Quick check if head sha matches
+                if pr.head_sha == query:
+                    result.append(FakeIssueSearchResult(FakeIssue(pr)))
+                    continue
+
+                # If head sha doesn't match it still could be in the pr history
+                repo = pr._getRepo()
+                commits = repo.iter_commits(
+                    '%s...%s' % (pr.branch, pr.head_sha))
+                for commit in commits:
+                    if commit.hexsha == query:
+                        result.append(FakeIssueSearchResult(FakeIssue(pr)))
+                        continue
+
+            return result
 
         # Non-SHA queries are of the form:
         #
@@ -684,3 +868,17 @@ class FakeGithubClient(object):
                     break
 
         return iter(results)
+
+
+class FakeGithubEnterpriseClient(FakeGithubClient):
+
+    version = '2.21.0'
+
+    def __init__(self, url, session=None, verify=True):
+        super().__init__(session=session)
+
+    def meta(self):
+        data = {
+            'installed_version': self.version,
+        }
+        return data
