@@ -32,7 +32,7 @@ class FunctionalZuulStreamMixIn:
         ansible_remote = os.environ.get('ZUUL_REMOTE_IPV4')
         self.assertIsNotNone(ansible_remote)
 
-    def _run_job(self, job_name):
+    def _run_job(self, job_name, create=True):
         # Keep the jobdir around so we can inspect contents if an
         # assert fails. It will be cleaned up anyway as it is contained
         # in a tmp dir which gets cleaned up after the test.
@@ -40,32 +40,40 @@ class FunctionalZuulStreamMixIn:
 
         # Output extra ansible info so we might see errors.
         self.executor_server.verbose = True
-        conf = textwrap.dedent(
-            """
-            - job:
-                name: {job_name}
-                run: playbooks/{job_name}.yaml
-                ansible-version: {version}
-                vars:
-                  test_console_port: {console_port}
-                roles:
-                  - zuul: org/common-config
-                nodeset:
-                  nodes:
-                    - name: compute1
-                      label: whatever
-                    - name: controller
-                      label: whatever
-
-            - project:
-                check:
-                  jobs:
-                    - {job_name}
-            """.format(
-                job_name=job_name,
-                version=self.ansible_version,
-                console_port=self.log_console_port))
+        if create:
+            conf = textwrap.dedent(
+                """
+                - job:
+                    name: {job_name}
+                    run: playbooks/{job_name}.yaml
+                    ansible-version: {version}
+                    vars:
+                      test_console_port: {console_port}
+                    roles:
+                      - zuul: org/common-config
+                    nodeset:
+                      nodes:
+                        - name: compute1
+                          label: whatever
+                        - name: controller
+                          label: whatever
 
+                - project:
+                    check:
+                      jobs:
+                        - {job_name}
+                """.format(
+                    job_name=job_name,
+                    version=self.ansible_version,
+                    console_port=self.log_console_port))
+        else:
+            conf = textwrap.dedent(
+                """
+                - project:
+                    check:
+                      jobs:
+                        - {job_name}
+                """.format(job_name=job_name))
         file_dict = {'zuul.yaml': conf}
         A = self.fake_gerrit.addFakeChange('org/project', 'master', 'A',
                                            files=file_dict)
@@ -141,9 +149,6 @@ class FunctionalZuulStreamMixIn:
             self.assertLogLine(r'compute1 \| failed_in_loop2', text)
             self.assertLogLine(r'compute1 \| ok: Item: failed_in_loop2 '
                                r'Result: 1', text)
-            self.assertLogLine(r'localhost \| .*No such file or directory: .*'
-                               r'\'/local-shelltask/somewhere/'
-                               r'that/does/not/exist\'', text)
             self.assertLogLine(r'compute1 \| .*No such file or directory: .*'
                                r'\'/remote-shelltask/somewhere/'
                                r'that/does/not/exist\'', text)
@@ -153,13 +158,27 @@ class FunctionalZuulStreamMixIn:
             self.assertLogLine(
                 r'controller \| ok: Runtime: \d:\d\d:\d\d\.\d\d\d\d\d\d', text)
             self.assertLogLine('PLAY RECAP', text)
+            # NOTE(pabelanger): Ansible 2.8 added new stats
+            # skipped, rescued, ignored.
             self.assertLogLine(
-                r'controller \| ok: \d+ changed: \d+ unreachable: 0 failed: 1',
-                text)
+                r'controller \| ok: \d+ changed: \d+ unreachable: 0 failed: 0 '
+                'skipped: 0 rescued: 1 ignored: 0', text)
             self.assertLogLine(
                 r'RUN END RESULT_NORMAL: \[untrusted : review.example.com/'
                 r'org/project/playbooks/command.yaml@master]', text)
 
+        # Run a pre-defined job that is defined in a trusted repo to test
+        # localhost tasks.
+        job = self._run_job('command-localhost', create=False)
+        with self.jobLog(job):
+            build = self.history[-1]
+            self.assertEqual(build.result, 'SUCCESS')
+
+            text = self._get_job_output(build)
+            self.assertLogLine(r'localhost \| .*No such file or directory: .*'
+                               r'\'/local-shelltask/somewhere/'
+                               r'that/does/not/exist\'', text)
+
     def test_module_exception(self):
         job = self._run_job('module_failure_exception')
         with self.jobLog(job):
@@ -193,14 +212,6 @@ class FunctionalZuulStreamMixIn:
             self.assertLogLine(regex, text)
 
 
-class TestZuulStream27(AnsibleZuulTestCase, FunctionalZuulStreamMixIn):
-    ansible_version = '2.7'
-
-    def setUp(self):
-        super().setUp()
-        self._setUp()
-
-
 class TestZuulStream28(AnsibleZuulTestCase, FunctionalZuulStreamMixIn):
     ansible_version = '2.8'
 
@@ -208,79 +219,10 @@ class TestZuulStream28(AnsibleZuulTestCase, FunctionalZuulStreamMixIn):
         super().setUp()
         self._setUp()
 
-    def test_command(self):
-        job = self._run_job('command')
-        with self.jobLog(job):
-            build = self.history[-1]
-            self.assertEqual(build.result, 'SUCCESS')
 
-            text = self._get_job_output(build)
-            self.assertLogLine(
-                r'RUN START: \[untrusted : review.example.com/org/project/'
-                r'playbooks/command.yaml@master\]', text)
-            self.assertLogLine(r'PLAY \[all\]', text)
-            self.assertLogLine(
-                r'Ansible version={}'.format(self.ansible_version), text)
-            self.assertLogLine(r'TASK \[Show contents of first file\]', text)
-            self.assertLogLine(r'controller \| command test one', text)
-            self.assertLogLine(
-                r'controller \| ok: Runtime: \d:\d\d:\d\d\.\d\d\d\d\d\d', text)
-            self.assertLogLine(r'TASK \[Show contents of second file\]', text)
-            self.assertLogLine(r'compute1 \| command test two', text)
-            self.assertLogLine(r'controller \| command test two', text)
-            self.assertLogLine(r'compute1 \| This is a rescue task', text)
-            self.assertLogLine(r'controller \| This is a rescue task', text)
-            self.assertLogLine(r'compute1 \| This is an always task', text)
-            self.assertLogLine(r'controller \| This is an always task', text)
-            self.assertLogLine(r'compute1 \| This is a handler', text)
-            self.assertLogLine(r'controller \| This is a handler', text)
-            self.assertLogLine(r'controller \| First free task', text)
-            self.assertLogLine(r'controller \| Second free task', text)
-            self.assertLogLine(r'controller \| This is a shell task after an '
-                               'included role', text)
-            self.assertLogLine(r'compute1 \| This is a shell task after an '
-                               'included role', text)
-            self.assertLogLine(r'controller \| This is a command task after '
-                               'an included role', text)
-            self.assertLogLine(r'compute1 \| This is a command task after an '
-                               'included role', text)
-            self.assertLogLine(r'controller \| This is a shell task with '
-                               'delegate compute1', text)
-            self.assertLogLine(r'controller \| This is a shell task with '
-                               'delegate controller', text)
-            self.assertLogLine(r'compute1 \| item_in_loop1', text)
-            self.assertLogLine(r'compute1 \| ok: Item: item_in_loop1 '
-                               r'Runtime: \d:\d\d:\d\d\.\d\d\d\d\d\d', text)
-            self.assertLogLine(r'compute1 \| item_in_loop2', text)
-            self.assertLogLine(r'compute1 \| ok: Item: item_in_loop2 '
-                               r'Runtime: \d:\d\d:\d\d\.\d\d\d\d\d\d', text)
-            self.assertLogLine(r'compute1 \| failed_in_loop1', text)
-            self.assertLogLine(r'compute1 \| ok: Item: failed_in_loop1 '
-                               r'Result: 1', text)
-            self.assertLogLine(r'compute1 \| failed_in_loop2', text)
-            self.assertLogLine(r'compute1 \| ok: Item: failed_in_loop2 '
-                               r'Result: 1', text)
-            self.assertLogLine(r'localhost \| .*No such file or directory: .*'
-                               r'\'/local-shelltask/somewhere/'
-                               r'that/does/not/exist\'', text)
-            self.assertLogLine(r'compute1 \| .*No such file or directory: .*'
-                               r'\'/remote-shelltask/somewhere/'
-                               r'that/does/not/exist\'', text)
-            self.assertLogLine(r'controller \| .*No such file or directory: .*'
-                               r'\'/remote-shelltask/somewhere/'
-                               r'that/does/not/exist\'', text)
-            self.assertLogLine(
-                r'controller \| ok: Runtime: \d:\d\d:\d\d\.\d\d\d\d\d\d', text)
-            self.assertLogLine('PLAY RECAP', text)
-            # NOTE(pabelanger): Ansible 2.8 added new stats
-            # skipped, rescued, ignored.
-            self.assertLogLine(
-                r'controller \| ok: \d+ changed: \d+ unreachable: 0 failed: 0 '
-                'skipped: 0 rescued: 1 ignored: 0', text)
-            self.assertLogLine(
-                r'RUN END RESULT_NORMAL: \[untrusted : review.example.com/'
-                r'org/project/playbooks/command.yaml@master]', text)
-
-
-class TestZuulStream29(TestZuulStream28):
+class TestZuulStream29(AnsibleZuulTestCase, FunctionalZuulStreamMixIn):
     ansible_version = '2.9'
+
+    def setUp(self):
+        super().setUp()
+        self._setUp()
