@@ -113,6 +113,50 @@ class TestTenantSimple(TenantParserTestCase):
             {'registry': 'registry.example.org', 'image_name': 'foo'},
         ])
 
+    def test_deny_localhost_nodeset(self):
+        in_repo_conf = textwrap.dedent(
+            """
+            - nodeset:
+                name: localhost
+                nodes:
+                  - name: localhost
+                    label: ubuntu
+            """)
+        file_dict = {'zuul.yaml': in_repo_conf}
+        A = self.fake_gerrit.addFakeChange('org/project1', 'master', 'A',
+                                           files=file_dict)
+        self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+
+        # No job should have run due to the change introducing a config error
+        self.assertHistory([])
+        self.assertTrue(A.reported)
+        self.assertTrue("Nodes named 'localhost' are not allowed."
+                        in A.messages[0])
+
+        in_repo_conf = textwrap.dedent(
+            """
+            - nodeset:
+                name: localhost-group
+                nodes:
+                  - name: ubuntu
+                    label: ubuntu
+                groups:
+                  - name: localhost
+                    nodes: ubuntu
+            """)
+        file_dict = {'zuul.yaml': in_repo_conf}
+        B = self.fake_gerrit.addFakeChange('org/project1', 'master', 'A',
+                                           files=file_dict)
+        self.fake_gerrit.addEvent(B.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+
+        # No job should have run due to the change introducing a config error
+        self.assertHistory([])
+        self.assertTrue(B.reported)
+        self.assertTrue("Groups named 'localhost' are not allowed."
+                        in B.messages[0])
+
 
 class TestTenantOverride(TenantParserTestCase):
     tenant_config_file = 'config/tenant-parser/override.yaml'
@@ -390,8 +434,13 @@ class TestTenantConfigBranches(ZuulTestCase):
         self.log.debug('Creating branches')
         self.create_branch('common-config', 'stable')
         self.create_branch('common-config', 'feat_x')
-
-        self.scheds.execute(lambda app: app.sched.reconfigure(app.config))
+        self.fake_gerrit.addEvent(
+            self.fake_gerrit.getFakeBranchCreatedEvent(
+                'common-config', 'stable'))
+        self.fake_gerrit.addEvent(
+            self.fake_gerrit.getFakeBranchCreatedEvent(
+                'common-config', 'feat_x'))
+        self.waitUntilSettled()
 
         # Job must be defined in master
         self._validate_job(common_job, 'master')
@@ -526,37 +575,55 @@ class TestUnparsedConfigCache(ZuulTestCase):
     tenant_config_file = 'config/single-tenant/main.yaml'
 
     def test_config_caching(self):
-        cache = self.scheds.first.sched. unparsed_config_cache
-        tenant = self.scheds.first.sched.abide.tenants["tenant-one"]
+        sched = self.scheds.first.sched
+        cache = sched.unparsed_config_cache
+        tenant = sched.abide.tenants["tenant-one"]
 
         common_cache = cache.getFilesCache("review.example.com/common-config",
                                            "master")
+        upb_common_cache = sched.abide.getUnparsedBranchCache(
+            "review.example.com/common-config", "master")
         tpc = tenant.project_configs["review.example.com/common-config"]
-        self.assertTrue(common_cache.isValidFor(tpc, cache_ltime=-1))
+        self.assertTrue(common_cache.isValidFor(tpc, min_ltime=-1))
         self.assertEqual(len(common_cache), 1)
         self.assertIn("zuul.yaml", common_cache)
         self.assertTrue(len(common_cache["zuul.yaml"]) > 0)
+        self.assertEqual(upb_common_cache.ltime, common_cache.ltime)
 
         project_cache = cache.getFilesCache("review.example.com/org/project",
                                             "master")
+        upb_project_cache = sched.abide.getUnparsedBranchCache(
+            "review.example.com/org/project", "master")
         # Cache of org/project should be valid but empty (no in-repo config)
         tpc = tenant.project_configs["review.example.com/org/project"]
-        self.assertTrue(project_cache.isValidFor(tpc, cache_ltime=-1))
+        self.assertTrue(project_cache.isValidFor(tpc, min_ltime=-1))
         self.assertEqual(len(project_cache), 0)
+        self.assertEqual(upb_project_cache.ltime, project_cache.ltime)
 
     def test_cache_use(self):
         sched = self.scheds.first.sched
-        # Stop cleanup thread so it's not removing projects from the cache
-        # during the test.
-        sched.cleanup_stop.set()
-        sched.cleanup_thread.join()
+        # Stop cleanup jobs so it's not removing projects from
+        # the cache during the test.
+        sched.apsched.shutdown()
         tenant = sched.abide.tenants['tenant-one']
         _, project = tenant.getProject('org/project2')
 
+        cache = self.scheds.first.sched.unparsed_config_cache
+        files_cache = cache.getFilesCache(
+            "review.example.com/org/project2", "master")
+        zk_initial_ltime = files_cache.ltime
+        upb_cache = sched.abide.getUnparsedBranchCache(
+            "review.example.com/org/project2", "master")
+        self.assertEqual(zk_initial_ltime, upb_cache.ltime)
+
         # Get the current ltime from Zookeeper and run a full reconfiguration,
         # so that we know all items in the cache have a larger ltime.
-        ltime = self.getCurrentLtime()
+        ltime = self.zk_client.getCurrentLtime()
         self.scheds.first.fullReconfigure()
+        self.assertGreater(files_cache.ltime, zk_initial_ltime)
+        upb_cache = sched.abide.getUnparsedBranchCache(
+            "review.example.com/org/project2", "master")
+        self.assertEqual(files_cache.ltime, upb_cache.ltime)
 
         # Clear the unparsed branch cache so all projects (except for
         # org/project2) are retrieved from the cache in Zookeeper.
@@ -568,14 +635,41 @@ class TestUnparsedConfigCache(ZuulTestCase):
         event = model.TenantReconfigureEvent(
             tenant.name, project.canonical_name, branch_name=None)
         event.zuul_event_ltime = ltime
-        sched.management_events.put(event, needs_result=False)
+        sched.management_events[tenant.name].put(event, needs_result=False)
         self.waitUntilSettled()
 
-        # As the cache should be valid, we only expect a cat job for
-        # org/project2
+        # As the cache should be valid (cache ltime of org/project2 newer than
+        # event ltime) we don't expect any cat jobs.
         cat_jobs = [job for job in self.gearman_server.jobs_history
                     if job.name == b"merger:cat"]
-        self.assertEqual(len(cat_jobs), 1)
+        self.assertEqual(len(cat_jobs), 0)
+
+        # Set canary value so we can detect if the configloader used
+        # the cache in Zookeeper (it shouldn't).
+        common_cache = cache.getFilesCache("review.example.com/common-config",
+                                           "master")
+        common_cache.setValidFor({"CANARY"}, set(), common_cache.ltime)
+        self.gearman_server.jobs_history.clear()
+
+        # Create a tenant reconfiguration event with a known ltime that is
+        # smaller than the ltime of the items in the cache.
+        event = model.TenantReconfigureEvent(
+            tenant.name, project.canonical_name, branch_name=None)
+        event.zuul_event_ltime = ltime
+        sched.management_events[tenant.name].put(event, needs_result=False)
+        self.waitUntilSettled()
+
+        upb_cache = sched.abide.getUnparsedBranchCache(
+            "review.example.com/common-config", "master")
+        self.assertEqual(common_cache.ltime, upb_cache.ltime)
+        self.assertNotIn("CANARY", upb_cache.extra_files_searched)
+
+        # As the cache should be valid (cache ltime of org/project2 newer than
+        # event ltime) we don't expect any cat jobs.
+        cat_jobs = [job for job in self.gearman_server.jobs_history
+                    if job.name == b"merger:cat"]
+        self.assertEqual(len(cat_jobs), 0)
+        sched.apsched.start()
 
 
 class TestAuthorizationRuleParser(ZuulTestCase):
