@@ -21,9 +21,9 @@ import types
 import sqlalchemy as sa
 
 import zuul
-from zuul.lib.yamlutil import yaml
+from zuul.lib import yamlutil
 from tests.base import ZuulTestCase, FIXTURE_DIR, \
-    PostgresqlSchemaFixture, MySQLSchemaFixture, ZuulDBTestCase, \
+    PostgresqlSchemaFixture, MySQLSchemaFixture, \
     BaseTestCase, AnsibleZuulTestCase
 
 
@@ -67,7 +67,7 @@ class TestConnections(ZuulTestCase):
                          'civoter')
 
 
-class TestSQLConnectionMysql(ZuulDBTestCase):
+class TestSQLConnectionMysql(ZuulTestCase):
     config_file = 'zuul-sql-driver-mysql.conf'
     tenant_config_file = 'config/sql-driver/main.yaml'
     expected_table_prefix = ''
@@ -126,16 +126,15 @@ class TestSQLConnectionMysql(ZuulDBTestCase):
     def test_sql_results(self):
         "Test results are entered into an sql table"
 
-        def check_results(connection_name):
+        def check_results():
             # Grab the sa tables
             tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
-            reporter = _get_reporter_from_connection_name(
-                tenant.layout.pipelines['check'].success_actions,
-                connection_name
-            )
-
-            conn = self.scheds.first.connections.connections[connection_name].\
+            pipeline = tenant.layout.pipelines['check']
+            reporter = self.scheds.first.connections.getSqlReporter(
+                pipeline)
+            conn = self.scheds.first.connections.getSqlConnection().\
                 engine.connect()
+
             result = conn.execute(
                 sa.sql.select([reporter.connection.zuul_buildset_table]))
 
@@ -229,22 +228,21 @@ class TestSQLConnectionMysql(ZuulDBTestCase):
         self.orderedRelease()
         self.waitUntilSettled()
 
-        check_results('database')
+        check_results()
 
     def test_sql_results_retry_builds(self):
         "Test that retry results are entered into an sql table correctly"
 
         # Check the results
-        def check_results(connection_name):
+        def check_results():
             # Grab the sa tables
             tenant = self.scheds.first.sched.abide.tenants.get("tenant-one")
-            reporter = _get_reporter_from_connection_name(
-                tenant.layout.pipelines["check"].success_actions,
-                connection_name
-            )
+            pipeline = tenant.layout.pipelines['check']
+            reporter = self.scheds.first.connections.getSqlReporter(
+                pipeline)
 
-            with self.scheds.first.connections.connections[connection_name]\
-                    .engine.connect() as conn:
+            with self.scheds.first.connections.getSqlConnection().\
+                    engine.connect() as conn:
 
                 result = conn.execute(
                     sa.sql.select([reporter.connection.zuul_buildset_table])
@@ -282,13 +280,13 @@ class TestSQLConnectionMysql(ZuulDBTestCase):
             self.assertEqual('project-test1', buildset0_builds[1]['job_name'])
             self.assertEqual('RETRY', buildset0_builds[1]['result'])
             self.assertFalse(buildset0_builds[1]['final'])
-            self.assertEqual('project-test1', buildset0_builds[2]['job_name'])
-            self.assertEqual('SUCCESS', buildset0_builds[2]['result'])
-            self.assertTrue(buildset0_builds[2]['final'])
+            self.assertEqual('project-test2', buildset0_builds[2]['job_name'])
+            self.assertEqual('RETRY', buildset0_builds[2]['result'])
+            self.assertFalse(buildset0_builds[2]['final'])
 
-            self.assertEqual('project-test2', buildset0_builds[3]['job_name'])
-            self.assertEqual('RETRY', buildset0_builds[3]['result'])
-            self.assertFalse(buildset0_builds[3]['final'])
+            self.assertEqual('project-test1', buildset0_builds[3]['job_name'])
+            self.assertEqual('SUCCESS', buildset0_builds[3]['result'])
+            self.assertTrue(buildset0_builds[3]['final'])
             self.assertEqual('project-test2', buildset0_builds[4]['job_name'])
             self.assertEqual('SUCCESS', buildset0_builds[4]['result'])
             self.assertTrue(buildset0_builds[4]['final'])
@@ -309,7 +307,7 @@ class TestSQLConnectionMysql(ZuulDBTestCase):
         self.orderedRelease()
         self.waitUntilSettled()
 
-        check_results('database')
+        check_results()
 
     def test_multiple_sql_connections(self):
         "Test putting results in different databases"
@@ -326,14 +324,13 @@ class TestSQLConnectionMysql(ZuulDBTestCase):
 
         def check_results(connection_name_1, connection_name_2):
             # Grab the sa tables for resultsdb
-            tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
-            reporter1 = _get_reporter_from_connection_name(
-                tenant.layout.pipelines['check'].success_actions,
-                connection_name_1
-            )
+            tenant = self.scheds.first.sched.abide.tenants.get("tenant-one")
+            pipeline = tenant.layout.pipelines['check']
+            reporter1 = self.scheds.first.connections.getSqlReporter(
+                pipeline)
 
-            conn = self.scheds.first.connections.\
-                connections[connection_name_1].engine.connect()
+            conn = self.scheds.first.connections.getSqlConnection().\
+                engine.connect()
             buildsets_resultsdb = conn.execute(sa.sql.select(
                 [reporter1.connection.zuul_buildset_table])).fetchall()
             # Should have been 2 buildset reported to the resultsdb (both
@@ -350,13 +347,6 @@ class TestSQLConnectionMysql(ZuulDBTestCase):
             self.assertEqual(
                 'Build succeeded.', buildsets_resultsdb[0]['message'])
 
-            # Grab the sa tables for resultsdb_mysql_failures
-            reporter2 = _get_reporter_from_connection_name(
-                tenant.layout.pipelines['check'].failure_actions,
-                connection_name_2
-            )
-            self.assertIsNone(reporter2)  # Explicit SQL reporters are ignored
-
             buildsets_resultsdb_failures = conn.execute(sa.sql.select(
                 [reporter1.connection.zuul_buildset_table])).fetchall()
             # The failure db should only have 1 buildset failed
@@ -549,17 +539,16 @@ class TestMultipleGerrits(ZuulTestCase):
 class TestConnectionsMerger(ZuulTestCase):
     config_file = 'zuul-connections-merger.conf'
     tenant_config_file = 'config/single-tenant/main.yaml'
-    source_only = True
 
     def test_connections_merger(self):
         "Test merger only configures source connections"
 
-        self.assertIn("gerrit", self.scheds.first.connections.connections)
-        self.assertIn("github", self.scheds.first.connections.connections)
-        self.assertNotIn("smtp", self.scheds.first.connections.connections)
-        self.assertNotIn("sql", self.scheds.first.connections.connections)
-        self.assertNotIn("timer", self.scheds.first.connections.connections)
-        self.assertNotIn("zuul", self.scheds.first.connections.connections)
+        self.assertIn("gerrit", self.executor_server.connections.connections)
+        self.assertIn("github", self.executor_server.connections.connections)
+        self.assertNotIn("smtp", self.executor_server.connections.connections)
+        self.assertNotIn("sql", self.executor_server.connections.connections)
+        self.assertNotIn("timer", self.executor_server.connections.connections)
+        self.assertNotIn("zuul", self.executor_server.connections.connections)
 
 
 class TestConnectionsCgit(ZuulTestCase):
@@ -643,15 +632,20 @@ class TestMQTTConnection(ZuulTestCase):
         self.assertEquals(test_job['result'], 'SUCCESS')
         self.assertEquals(test_job['dependencies'], [])
         self.assertEquals(test_job['artifacts'], [artifact])
-        # Both log- and web-url should point to the same URL which is specified
-        # in the build result data under zuul.log_url.
         self.assertEquals(test_job['log_url'], 'some-log-url/')
-        self.assertEquals(test_job['web_url'], 'some-log-url')
+        build_id = test_job["uuid"]
+        self.assertEquals(
+            test_job["web_url"],
+            "https://tenant.example.com/t/tenant-one/build/{}".format(
+                build_id
+            ),
+        )
         self.assertIn('execute_time', test_job)
         self.assertIn('timestamp', mqtt_payload)
         self.assertIn('enqueue_time', mqtt_payload)
         self.assertIn('trigger_time', mqtt_payload)
         self.assertIn('zuul_event_id', mqtt_payload)
+        self.assertIn('uuid', mqtt_payload)
         self.assertEquals(dependent_test_job['dependencies'], ['test'])
 
     def test_mqtt_invalid_topic(self):
@@ -677,51 +671,6 @@ class TestMQTTConnection(ZuulTestCase):
                       "A should report a syntax error")
 
 
-class TestMQTTConnectionBuildPage(ZuulTestCase):
-    config_file = "zuul-mqtt-driver.conf"
-    tenant_config_file = "config/mqtt-driver-report-build-page/main.yaml"
-
-    def test_mqtt_reporter(self):
-        "Test the MQTT reporter with 'report-build-page' enabled"
-
-        # Add a sucess result
-        A = self.fake_gerrit.addFakeChange("org/project", "master", "A")
-        self.executor_server.returnData(
-            "test", A, {"zuul": {"log_url": "some-log-url"}}
-        )
-        self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
-        self.waitUntilSettled()
-
-        success_event = self.mqtt_messages.pop()
-
-        self.assertEquals(
-            success_event.get("topic"),
-            "tenant-one/zuul_buildset/check/org/project/master",
-        )
-
-        mqtt_payload = success_event["msg"]
-        self.assertEquals(mqtt_payload["project"], "org/project")
-        self.assertEquals(mqtt_payload["branch"], "master")
-        self.assertEquals(mqtt_payload["buildset"]["result"], "SUCCESS")
-
-        builds = mqtt_payload["buildset"]["builds"]
-        test_job = [b for b in builds if b["job_name"] == "test"][0]
-        self.assertEquals(test_job["job_name"], "test")
-        self.assertEquals(test_job["result"], "SUCCESS")
-
-        build_id = test_job["uuid"]
-        # When report-build-page is enabled, the log_url should still point to
-        # the URL that is specified in the build result data under
-        # zuul.log_url. The web_url will instead point to the builds page.
-        self.assertEquals(test_job["log_url"], "some-log-url/")
-        self.assertEquals(
-            test_job["web_url"],
-            "https://tenant.example.com/t/tenant-one/build/{}".format(
-                build_id
-            ),
-        )
-
-
 class TestElasticsearchConnection(AnsibleZuulTestCase):
     config_file = 'zuul-elastic-driver.conf'
     tenant_config_file = 'config/elasticsearch-driver/main.yaml'
@@ -731,7 +680,8 @@ class TestElasticsearchConnection(AnsibleZuulTestCase):
         build = self.getJobFromHistory(job)
         for pb in getattr(build.jobdir, pbtype):
             if pb.secrets_content:
-                secrets.append(yaml.safe_load(pb.secrets_content))
+                secrets.append(
+                    yamlutil.ansible_unsafe_load(pb.secrets_content))
             else:
                 secrets.append({})
         return secrets
