@@ -239,7 +239,7 @@ class TestGerritToGithubCRD(ZuulTestCase):
     def test_crd_check(self):
         "Test cross-repo dependencies in independent pipelines"
         self.executor_server.hold_jobs_in_build = True
-        self.gearman_server.hold_jobs_in_queue = True
+        self.hold_jobs_in_queue = True
         A = self.fake_gerrit.addFakeChange('gerrit/project1', 'master', 'A')
         B = self.fake_github.openFakePullRequest(
             'github/project2', 'master', 'B')
@@ -251,8 +251,8 @@ class TestGerritToGithubCRD(ZuulTestCase):
         self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
         self.waitUntilSettled()
 
-        self.gearman_server.hold_jobs_in_queue = False
-        self.gearman_server.release()
+        self.hold_jobs_in_queue = False
+        self.executor_api.release()
         self.waitUntilSettled()
 
         self.executor_server.release('.*-merge')
@@ -334,7 +334,7 @@ class TestGerritToGithubCRD(ZuulTestCase):
     def _test_crd_check_reconfiguration(self, project1, project2):
         "Test cross-repo dependencies re-enqueued in independent pipelines"
 
-        self.gearman_server.hold_jobs_in_queue = True
+        self.hold_jobs_in_queue = True
         A = self.fake_gerrit.addFakeChange('gerrit/project1', 'master', 'A')
         B = self.fake_github.openFakePullRequest(
             'github/project2', 'master', 'B')
@@ -359,8 +359,8 @@ class TestGerritToGithubCRD(ZuulTestCase):
         self.assertFalse(first_item.live)
         self.assertTrue(queue.queue[1].live)
 
-        self.gearman_server.hold_jobs_in_queue = False
-        self.gearman_server.release()
+        self.hold_jobs_in_queue = False
+        self.executor_api.release()
         self.waitUntilSettled()
 
         self.assertEqual(A.data['status'], 'NEW')
@@ -694,7 +694,7 @@ class TestGithubToGerritCRD(ZuulTestCase):
     def test_crd_check(self):
         "Test cross-repo dependencies in independent pipelines"
         self.executor_server.hold_jobs_in_build = True
-        self.gearman_server.hold_jobs_in_queue = True
+        self.hold_jobs_in_queue = True
         A = self.fake_github.openFakePullRequest('github/project2', 'master',
                                                  'A')
         B = self.fake_gerrit.addFakeChange(
@@ -706,8 +706,8 @@ class TestGithubToGerritCRD(ZuulTestCase):
         self.fake_github.emitEvent(A.getPullRequestEditedEvent())
         self.waitUntilSettled()
 
-        self.gearman_server.hold_jobs_in_queue = False
-        self.gearman_server.release()
+        self.hold_jobs_in_queue = False
+        self.executor_api.release()
         self.waitUntilSettled()
 
         self.executor_server.release('.*-merge')
@@ -787,7 +787,7 @@ class TestGithubToGerritCRD(ZuulTestCase):
     def _test_crd_check_reconfiguration(self, project1, project2):
         "Test cross-repo dependencies re-enqueued in independent pipelines"
 
-        self.gearman_server.hold_jobs_in_queue = True
+        self.hold_jobs_in_queue = True
         A = self.fake_github.openFakePullRequest('github/project2', 'master',
                                                  'A')
         B = self.fake_gerrit.addFakeChange(
@@ -812,8 +812,8 @@ class TestGithubToGerritCRD(ZuulTestCase):
         self.assertFalse(first_item.live)
         self.assertTrue(queue.queue[1].live)
 
-        self.gearman_server.hold_jobs_in_queue = False
-        self.gearman_server.release()
+        self.hold_jobs_in_queue = False
+        self.executor_api.release()
         self.waitUntilSettled()
 
         self.assertFalse(A.is_merged)
