@@ -21,11 +21,8 @@ import os
 import time
 from unittest import mock
 
-import zuul.executor.server
-import zuul.model
-import gear
-
 from tests.base import (
+    BaseTestCase,
     ZuulTestCase,
     AnsibleZuulTestCase,
     FIXTURE_DIR,
@@ -35,7 +32,9 @@ from tests.base import (
 
 from zuul.executor.sensors.startingbuilds import StartingBuildsSensor
 from zuul.executor.sensors.ram import RAMSensor
+from zuul.executor.server import AnsibleJob, JobDir, squash_variables
 from zuul.lib.ansible import AnsibleManager
+from zuul.model import BuildRequest, NodeSet, Group
 
 
 class TestExecutorRepos(ZuulTestCase):
@@ -436,63 +435,107 @@ class TestAnsibleJob(ZuulTestCase):
     def setUp(self):
         super(TestAnsibleJob, self).setUp()
         ansible_version = AnsibleManager().default_version
-        args = '{"ansible_version": "%s"}' % ansible_version
-        job = gear.TextJob('executor:execute', args, unique='test')
-        self.test_job = zuul.executor.server.AnsibleJob(self.executor_server,
-                                                        job)
+        params = {
+            "ansible_version": ansible_version,
+            "zuul_event_id": 0,
+            "nodeset": {
+                "name": "dummy-node",
+                "node_request_id": 0,
+                "nodes": [],
+                "groups": [],
+            },
+        }
+        build_request = BuildRequest(
+            "test",
+            state=None,
+            precedence=200,
+            zone=None,
+            tenant_name=None,
+            pipeline_name=None,
+            event_id='1',
+        )
 
-    def test_getHostList_host_keys(self):
+        self.test_job = AnsibleJob(self.executor_server, build_request, params)
+        self.test_job.jobdir = JobDir(self.executor_server.jobdir_root,
+                                      self.executor_server.keep_jobdir,
+                                      str(build_request.uuid))
+
+    def test_prepareNodes_host_keys(self):
         # Test without connection_port set
         node = {'name': 'fake-host',
+                'label': 'fake-label',
+                'state': 'ready',
+                'cloud': 'fake',
                 'host_keys': ['fake-host-key'],
                 'interface_ip': 'localhost'}
-        keys = self.test_job.getHostList({'nodes': [node],
-                                          'host_vars': {},
-                                          'vars': {},
-                                          'groups': [],
-                                          })[0]['host_keys']
+        nodeset = {
+            "name": "dummy-node",
+            "node_request_id": 0,
+            "nodes": [node],
+            "groups": [],
+        }
+        self.test_job.nodeset = NodeSet.fromDict(nodeset)
+        self.test_job.prepareNodes({'host_vars': {},
+                                    'vars': {},
+                                    'groups': [],
+                                    })
+        keys = self.test_job.host_list[0]['host_keys']
         self.assertEqual(keys[0], 'localhost fake-host-key')
 
         # Test with custom connection_port set
         node['connection_port'] = 22022
-        keys = self.test_job.getHostList({'nodes': [node],
-                                          'host_vars': {},
-                                          'vars': {},
-                                          'groups': [],
-                                          })[0]['host_keys']
+        self.test_job.nodeset = NodeSet.fromDict(nodeset)
+        self.test_job.prepareNodes({'host_vars': {},
+                                    'vars': {},
+                                    'groups': [],
+                                    })
+        keys = self.test_job.host_list[0]['host_keys']
         self.assertEqual(keys[0], '[localhost]:22022 fake-host-key')
 
         # Test with no host keys
         node['host_keys'] = []
-        host = self.test_job.getHostList({'nodes': [node],
-                                          'host_vars': {},
-                                          'vars': {},
-                                          'groups': [],
-                                          })[0]
+        self.test_job.nodeset = NodeSet.fromDict(nodeset)
+        self.test_job.prepareNodes({'nodes': [node],
+                                    'host_vars': {},
+                                    'vars': {},
+                                    'groups': [],
+                                    })
+        host = self.test_job.host_list[0]
         self.assertEqual(host['host_keys'], [])
         self.assertEqual(
             host['host_vars']['ansible_ssh_common_args'],
             '-o StrictHostKeyChecking=false')
 
-    def test_getHostList_shell_type(self):
+    def test_prepareNodes_shell_type(self):
         # Test without shell type set
         node = {'name': 'fake-host',
+                'label': 'fake-label',
+                'state': 'ready',
+                'cloud': 'fake',
                 'host_keys': ['fake-host-key'],
                 'interface_ip': 'localhost'}
-        host = self.test_job.getHostList({'nodes': [node],
-                                          'host_vars': {},
-                                          'vars': {},
-                                          'groups': [],
-                                          })[0]
+        nodeset = {
+            "name": "dummy-node",
+            "node_request_id": 0,
+            "nodes": [node],
+            "groups": [],
+        }
+        self.test_job.nodeset = NodeSet.fromDict(nodeset)
+        self.test_job.prepareNodes({'host_vars': {},
+                                    'vars': {},
+                                    'groups': [],
+                                    })
+        host = self.test_job.host_list[0]
         self.assertNotIn('ansible_shell_type', host['host_vars'])
 
         # Test with custom shell type set.
         node['shell_type'] = 'cmd'
-        host = self.test_job.getHostList({'nodes': [node],
-                                          'host_vars': {},
-                                          'vars': {},
-                                          'groups': [],
-                                          })[0]
+        self.test_job.nodeset = NodeSet.fromDict(nodeset)
+        self.test_job.prepareNodes({'host_vars': {},
+                                    'vars': {},
+                                    'groups': [],
+                                    })
+        host = self.test_job.host_list[0]
         self.assertIn('ansible_shell_type', host['host_vars'])
         self.assertEqual(
             host['host_vars']['ansible_shell_type'],
@@ -957,3 +1000,63 @@ class TestExecutorExtraPackages(AnsibleZuulTestCase):
         self.assertFalse(ansible_manager.validate())
         ansible_manager.install()
         self.assertTrue(ansible_manager.validate())
+
+
+class TestVarSquash(BaseTestCase):
+    def test_squash_variables(self):
+        # Test that we correctly squash job variables
+        nodeset = NodeSet()
+        nodes = [
+            {'name': 'node1', 'host_vars': {
+                'host': 'node1_host',
+                'extra': 'node1_extra',
+            }},
+            {'name': 'node2', 'host_vars': {
+                'host': 'node2_host',
+                'extra': 'node2_extra',
+            }},
+        ]
+        nodeset.addGroup(Group('group1', ['node1']))
+        nodeset.addGroup(Group('group2', ['node2']))
+        groupvars = {
+            'group1': {
+                'host': 'group1_host',
+                'group': 'group1_group',
+                'extra': 'group1_extra',
+            },
+            'group2': {
+                'host': 'group2_host',
+                'group': 'group2_group',
+                'extra': 'group2_extra',
+            },
+            'all': {
+                'all2': 'groupvar_all2',
+            }
+        }
+        jobvars = {
+            'host': 'jobvar_host',
+            'group': 'jobvar_group',
+            'all': 'jobvar_all',
+            'extra': 'jobvar_extra',
+        }
+        extravars = {
+            'extra': 'extravar_extra',
+        }
+        out = squash_variables(
+            nodes, nodeset, jobvars, groupvars, extravars)
+
+        expected = {
+            'node1': {
+                'all': 'jobvar_all',
+                'all2': 'groupvar_all2',
+                'group': 'group1_group',
+                'host': 'node1_host',
+                'extra': 'extravar_extra'},
+            'node2': {
+                'all': 'jobvar_all',
+                'all2': 'groupvar_all2',
+                'group': 'group2_group',
+                'host': 'node2_host',
+                'extra': 'extravar_extra'},
+        }
+        self.assertEqual(out, expected)
