@@ -27,13 +27,17 @@ import github3.exceptions
 
 from tests.fakegithub import FakeGithubEnterpriseClient
 from zuul.driver.github.githubconnection import GithubShaCache
+from zuul.zk.layout import LayoutState
 import zuul.rpcclient
+from zuul.lib import strings
 
 from tests.base import (AnsibleZuulTestCase, BaseTestCase,
                         ZuulGithubAppTestCase, ZuulTestCase,
                         simple_layout, random_sha1)
 from tests.base import ZuulWebFixture
 
+EMPTY_LAYOUT_STATE = LayoutState("", "", 0)
+
 
 class TestGithubDriver(ZuulTestCase):
     config_file = 'zuul-github-driver.conf'
@@ -64,7 +68,9 @@ class TestGithubDriver(ZuulTestCase):
         self.assertEqual('master', zuulvars['branch'])
         self.assertEquals('https://github.com/org/project/pull/1',
                           zuulvars['items'][0]['change_url'])
-        self.assertEqual(zuulvars["message"], "A\n\n{}".format(body))
+        expected = "A\n\n{}".format(body)
+        self.assertEqual(zuulvars["message"],
+                         strings.b64encode(expected))
         self.assertEqual(1, len(A.comments))
         self.assertThat(
             A.comments[0],
@@ -219,8 +225,8 @@ class TestGithubDriver(ZuulTestCase):
         self.waitUntilSettled()
 
         # Record previous tenant reconfiguration time
-        before = self.scheds.first.sched.tenant_last_reconfigured.get(
-            'tenant-one', 0)
+        before = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
 
         self.fake_github.emitEvent(
             self.fake_github.getPushEvent('org/project', 'refs/tags/newtag',
@@ -228,8 +234,8 @@ class TestGithubDriver(ZuulTestCase):
         self.waitUntilSettled()
 
         # Make sure the tenant hasn't been reconfigured due to the new tag
-        after = self.scheds.first.sched.tenant_last_reconfigured.get(
-            'tenant-one', 0)
+        after = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
         self.assertEqual(before, after)
 
         build_params = self.builds[0].parameters
@@ -965,8 +971,8 @@ class TestGithubDriver(ZuulTestCase):
             removed_files=removed_files)
 
         # record previous tenant reconfiguration time, which may not be set
-        old = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        old = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
         self.waitUntilSettled()
 
         if expected_cat_jobs is not None:
@@ -976,8 +982,8 @@ class TestGithubDriver(ZuulTestCase):
 
         self.fake_github.emitEvent(pevent)
         self.waitUntilSettled()
-        new = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        new = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
 
         if expect_reconfigure:
             # New timestamp should be greater than the old timestamp
@@ -1333,7 +1339,10 @@ class TestGithubUnprotectedBranches(ZuulTestCase):
         github = self.fake_github.getGithubClient()
         repo = github.repo_from_project('org/project2')
         repo._set_branch_protection('master', True)
-        self.scheds.execute(lambda app: app.sched.reconfigure(app.config))
+
+        pevent = self.fake_github.getPushEvent(project='org/project2',
+                                               ref='refs/heads/master')
+        self.fake_github.emitEvent(pevent)
         self.waitUntilSettled()
 
         tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
@@ -1431,14 +1440,14 @@ class TestGithubUnprotectedBranches(ZuulTestCase):
                                                modified_files=['zuul.yaml'])
 
         # record previous tenant reconfiguration time, which may not be set
-        old = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        old = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
         self.waitUntilSettled()
 
         self.fake_github.emitEvent(pevent)
         self.waitUntilSettled()
-        new = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        new = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
 
         # We don't expect a reconfiguration because the push was to an
         # unprotected branch
@@ -1451,8 +1460,8 @@ class TestGithubUnprotectedBranches(ZuulTestCase):
 
         self.fake_github.emitEvent(pevent)
         self.waitUntilSettled()
-        new = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        new = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
 
         # We now expect that zuul reconfigured itself
         self.assertLess(old, new)
@@ -1464,6 +1473,9 @@ class TestGithubUnprotectedBranches(ZuulTestCase):
         github = self.fake_github.getGithubClient()
         repo = github.repo_from_project('org/project2')
         repo._set_branch_protection('master', True)
+        self.fake_github.emitEvent(
+            self.fake_github.getPushEvent(
+                project='org/project2', ref='refs/heads/master'))
 
         A = self.fake_github.openFakePullRequest('org/project2', 'master', 'A')
         A.setMerged("merging A")
@@ -1471,13 +1483,15 @@ class TestGithubUnprotectedBranches(ZuulTestCase):
         # add a spare branch so that the project is not empty after master gets
         # deleted.
         repo._create_branch('feat-x')
+        self.fake_github.emitEvent(
+            self.fake_github.getPushEvent(
+                project='org/project2', ref='refs/heads/feat-x'))
 
-        self.scheds.execute(lambda app: app.sched.reconfigure(app.config))
         self.waitUntilSettled()
 
         # record previous tenant reconfiguration time, which may not be set
-        old = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        old = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
         self.waitUntilSettled()
 
         # Delete the branch
@@ -1490,8 +1504,8 @@ class TestGithubUnprotectedBranches(ZuulTestCase):
 
         self.fake_github.emitEvent(pevent)
         self.waitUntilSettled()
-        new = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        new = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
 
         # We now expect that zuul reconfigured itself as we deleted a protected
         # branch
@@ -1551,8 +1565,8 @@ class TestGithubUnprotectedBranches(ZuulTestCase):
             removed_files=removed_files)
 
         # record previous tenant reconfiguration time, which may not be set
-        old = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        old = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
         self.waitUntilSettled()
 
         if expected_cat_jobs is not None:
@@ -1562,8 +1576,8 @@ class TestGithubUnprotectedBranches(ZuulTestCase):
 
         self.fake_github.emitEvent(pevent)
         self.waitUntilSettled()
-        new = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        new = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
 
         if expect_reconfigure:
             # New timestamp should be greater than the old timestamp
@@ -1872,7 +1886,7 @@ class TestGithubAppDriver(ZuulGithubAppTestCase):
     @simple_layout("layouts/reporting-github.yaml", driver="github")
     def test_reporting_checks_api_dequeue(self):
         "Test that a dequeued change will be reported back to the check run"
-        project = "org/project3"
+        project = "org/project4"
         github = self.fake_github.getGithubClient(None)
 
         client = zuul.rpcclient.RPCClient(
@@ -1893,31 +1907,34 @@ class TestGithubAppDriver(ZuulGithubAppTestCase):
         self.assertEqual(1, len(check_runs))
         check_run = check_runs[0]
 
-        self.assertEqual("tenant-one/checks-api-reporting", check_run["name"])
+        self.assertEqual(
+            "tenant-one/checks-api-reporting-skipped", check_run["name"])
         self.assertEqual("in_progress", check_run["status"])
         self.assertThat(
             check_run["output"]["summary"],
-            MatchesRegex(r'.*Starting checks-api-reporting jobs.*', re.DOTALL)
+            MatchesRegex(
+                r'.*Starting checks-api-reporting-skipped jobs.*', re.DOTALL)
         )
 
         # Use the client to dequeue the pending change
         client.dequeue(
             tenant="tenant-one",
-            pipeline="checks-api-reporting",
-            project="org/project3",
+            pipeline="checks-api-reporting-skipped",
+            project="org/project4",
             change="{},{}".format(A.number, A.head_sha),
             ref=None,
         )
         self.waitUntilSettled()
 
-        # We should now have a cancelled check run for the head sha
+        # We should now have a skipped check run for the head sha
         check_runs = self.fake_github.getCommitChecks(project, A.head_sha)
         self.assertEqual(1, len(check_runs))
         check_run = check_runs[0]
 
-        self.assertEqual("tenant-one/checks-api-reporting", check_run["name"])
+        self.assertEqual(
+            "tenant-one/checks-api-reporting-skipped", check_run["name"])
         self.assertEqual("completed", check_run["status"])
-        self.assertEqual("cancelled", check_run["conclusion"])
+        self.assertEqual("skipped", check_run["conclusion"])
         self.assertThat(
             check_run["output"]["summary"],
             MatchesRegex(r'.*Build canceled.*', re.DOTALL)
