@@ -19,12 +19,16 @@ import yaml
 import socket
 
 import zuul.rpcclient
+from zuul.lib import strings
+from zuul.zk.layout import LayoutState
 
 from tests.base import random_sha1, simple_layout
 from tests.base import ZuulTestCase, ZuulWebFixture
 
 from testtools.matchers import MatchesRegex
 
+EMPTY_LAYOUT_STATE = LayoutState("", "", 0)
+
 
 class TestGitlabWebhook(ZuulTestCase):
     config_file = 'zuul-gitlab-driver.conf'
@@ -106,7 +110,7 @@ class TestGitlabDriver(ZuulTestCase):
         self.assertEqual('master', zuulvars['branch'])
         self.assertEquals('https://gitlab/org/project/merge_requests/1',
                           zuulvars['items'][0]['change_url'])
-        self.assertEqual(zuulvars["message"], description)
+        self.assertEqual(zuulvars["message"], strings.b64encode(description))
         self.assertEqual(2, len(self.history))
         self.assertEqual(2, len(A.notes))
         self.assertEqual(
@@ -293,12 +297,12 @@ class TestGitlabDriver(ZuulTestCase):
         event = self.fake_gitlab.getPushEvent(
             'org/project', branch='refs/heads/stable-1.0',
             before='0' * 40, after=newrev)
-        old = self.scheds.first.sched.tenant_last_reconfigured.get(
-            'tenant-one', 0)
+        old = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
         self.fake_gitlab.emitEvent(event)
         self.waitUntilSettled()
-        new = self.scheds.first.sched.tenant_last_reconfigured.get(
-            'tenant-one', 0)
+        new = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
         # New timestamp should be greater than the old timestamp
         self.assertLess(old, new)
         self.assertEqual(1, len(self.history))
@@ -382,8 +386,8 @@ class TestGitlabDriver(ZuulTestCase):
     def test_ref_updated_and_tenant_reconfigure(self):
 
         self.waitUntilSettled()
-        old = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        old = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
 
         zuul_yaml = [
             {'job': {
@@ -409,8 +413,8 @@ class TestGitlabDriver(ZuulTestCase):
         self.fake_gitlab.emitEvent(event)
         self.waitUntilSettled()
 
-        new = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        new = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
         # New timestamp should be greater than the old timestamp
         self.assertLess(old, new)
 
@@ -699,6 +703,61 @@ class TestGitlabDriver(ZuulTestCase):
         self.assertTrue(A.is_merged)
         self.assertTrue(B.is_merged)
 
+    @simple_layout('layouts/crd-gitlab.yaml', driver='gitlab')
+    def test_api_token(self):
+        tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
+        _, project = tenant.getProject('org/project1')
+
+        project_git_url = self.fake_gitlab.real_getGitUrl(project)
+        # cloneurl created from config 'server' should be used
+        # without credentials
+        self.assertEqual("https://gitlab/org/project1.git", project_git_url)
+
+    @simple_layout('layouts/crd-gitlab.yaml', driver='gitlab2')
+    def test_api_token_cloneurl(self):
+        tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
+        _, project = tenant.getProject('org/project1')
+
+        project_git_url = self.fake_gitlab2.real_getGitUrl(project)
+        # cloneurl from config file should be used as it defines token name and
+        # secret
+        self.assertEqual("http://myusername:2222@gitlab/org/project1.git",
+                         project_git_url)
+
+    @simple_layout('layouts/crd-gitlab.yaml', driver='gitlab3')
+    def test_api_token_name_cloneurl(self):
+        tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
+        _, project = tenant.getProject('org/project1')
+
+        project_git_url = self.fake_gitlab3.real_getGitUrl(project)
+        # cloneurl from config file should be used as it defines token name and
+        # secret, even if token name and token secret are defined
+        self.assertEqual("http://myusername:2222@gitlabthree/org/project1.git",
+                         project_git_url)
+
+    @simple_layout('layouts/crd-gitlab.yaml', driver='gitlab4')
+    def test_api_token_name(self):
+        tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
+        _, project = tenant.getProject('org/project1')
+
+        project_git_url = self.fake_gitlab4.real_getGitUrl(project)
+        # cloneurl is not set, generate one from token name, token secret and
+        # server
+        self.assertEqual("https://tokenname4:444@gitlabfour/org/project1.git",
+                         project_git_url)
+
+    @simple_layout('layouts/crd-gitlab.yaml', driver='gitlab5')
+    def test_api_token_name_cloneurl_server(self):
+        tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
+        _, project = tenant.getProject('org/project1')
+
+        project_git_url = self.fake_gitlab5.real_getGitUrl(project)
+        # cloneurl defines a url, without credentials. As token name is
+        # set, include token name and secret in cloneurl, 'server' is
+        # overwritten
+        self.assertEqual("http://tokenname5:555@gitlabfivvve/org/project1.git",
+                         project_git_url)
+
 
 class TestGitlabUnprotectedBranches(ZuulTestCase):
     config_file = 'zuul-gitlab-driver.conf'
@@ -722,7 +781,8 @@ class TestGitlabUnprotectedBranches(ZuulTestCase):
 
         # now enable branch protection and trigger reload
         self.fake_gitlab.protectBranch('org', 'project2', 'master')
-        self.scheds.execute(lambda app: app.sched.reconfigure(app.config))
+        pevent = self.fake_gitlab.getPushEvent(project='org/project2')
+        self.fake_gitlab.emitEvent(pevent)
         self.waitUntilSettled()
 
         tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
@@ -836,14 +896,14 @@ class TestGitlabUnprotectedBranches(ZuulTestCase):
                                                branch='refs/heads/master')
 
         # record previous tenant reconfiguration time, which may not be set
-        old = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        old = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
         self.waitUntilSettled()
 
         self.fake_gitlab.emitEvent(pevent)
         self.waitUntilSettled()
-        new = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        new = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
 
         # We don't expect a reconfiguration because the push was to an
         # unprotected branch
@@ -854,8 +914,8 @@ class TestGitlabUnprotectedBranches(ZuulTestCase):
 
         self.fake_gitlab.emitEvent(pevent)
         self.waitUntilSettled()
-        new = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        new = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
 
         # We now expect that zuul reconfigured itself
         self.assertLess(old, new)
@@ -865,6 +925,9 @@ class TestGitlabUnprotectedBranches(ZuulTestCase):
 
         # Prepare repo with an initial commit and enable branch protection
         self.fake_gitlab.protectBranch('org', 'project2', 'master')
+        self.fake_gitlab.emitEvent(
+            self.fake_gitlab.getPushEvent(
+                project='org/project2', branch='refs/heads/master'))
 
         A = self.fake_gitlab.openFakeMergeRequest('org/project2', 'master',
                                                   'A')
@@ -875,13 +938,14 @@ class TestGitlabUnprotectedBranches(ZuulTestCase):
         self.create_branch('org/project2', 'feat-x')
         self.fake_gitlab.protectBranch('org', 'project2', 'feat-x',
                                        protected=False)
-
-        self.scheds.execute(lambda app: app.sched.reconfigure(app.config))
+        self.fake_gitlab.emitEvent(
+            self.fake_gitlab.getPushEvent(
+                project='org/project2', branch='refs/heads/feat-x'))
         self.waitUntilSettled()
 
         # record previous tenant reconfiguration time, which may not be set
-        old = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        old = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
         self.waitUntilSettled()
 
         # Delete the branch
@@ -894,8 +958,8 @@ class TestGitlabUnprotectedBranches(ZuulTestCase):
 
         self.fake_gitlab.emitEvent(pevent)
         self.waitUntilSettled()
-        new = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        new = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
 
         # We now expect that zuul reconfigured itself as we deleted a protected
         # branch
@@ -954,8 +1018,8 @@ class TestGitlabUnprotectedBranches(ZuulTestCase):
             after=new_sha)
 
         # record previous tenant reconfiguration time, which may not be set
-        old = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        old = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
         self.waitUntilSettled()
 
         if expected_cat_jobs is not None:
@@ -965,8 +1029,8 @@ class TestGitlabUnprotectedBranches(ZuulTestCase):
 
         self.fake_gitlab.emitEvent(pevent)
         self.waitUntilSettled()
-        new = self.scheds.first.sched.tenant_last_reconfigured\
-            .get('tenant-one', 0)
+        new = self.scheds.first.sched.tenant_layout_state.get(
+            'tenant-one', EMPTY_LAYOUT_STATE)
 
         if expect_reconfigure:
             # New timestamp should be greater than the old timestamp
