@@ -51,6 +51,7 @@ class TestJob(BaseTestCase):
         self.tenant = model.Tenant('tenant')
         self.tenant.default_ansible_version = AnsibleManager().default_version
         self.layout = model.Layout(self.tenant)
+        self.tenant.layout = self.layout
         self.project = model.Project('project', self.source)
         self.context = model.SourceContext(self.project, 'master',
                                            'test', True)
@@ -204,13 +205,12 @@ class TestJob(BaseTestCase):
         change = model.Change(self.project)
         change.branch = 'master'
         item = queue.enqueueChange(change, None)
-        item.layout = self.layout
 
         self.assertTrue(base.changeMatchesBranch(change))
         self.assertTrue(python27.changeMatchesBranch(change))
         self.assertFalse(python27diablo.changeMatchesBranch(change))
 
-        item.freezeJobGraph()
+        item.freezeJobGraph(self.layout)
         self.assertEqual(len(item.getJobs()), 1)
         job = item.getJobs()[0]
         self.assertEqual(job.name, 'python27')
@@ -218,13 +218,12 @@ class TestJob(BaseTestCase):
 
         change.branch = 'stable/diablo'
         item = queue.enqueueChange(change, None)
-        item.layout = self.layout
 
         self.assertTrue(base.changeMatchesBranch(change))
         self.assertTrue(python27.changeMatchesBranch(change))
         self.assertTrue(python27diablo.changeMatchesBranch(change))
 
-        item.freezeJobGraph()
+        item.freezeJobGraph(self.layout)
         self.assertEqual(len(item.getJobs()), 1)
         job = item.getJobs()[0]
         self.assertEqual(job.name, 'python27')
@@ -267,12 +266,11 @@ class TestJob(BaseTestCase):
         change.branch = 'master'
         change.files = ['/COMMIT_MSG', 'ignored-file']
         item = queue.enqueueChange(change, None)
-        item.layout = self.layout
 
         self.assertTrue(base.changeMatchesFiles(change))
         self.assertFalse(python27.changeMatchesFiles(change))
 
-        item.freezeJobGraph()
+        item.freezeJobGraph(self.layout)
         self.assertEqual([], item.getJobs())
 
     def test_job_source_project(self):
@@ -340,7 +338,7 @@ class TestJob(BaseTestCase):
         with testtools.ExpectedException(
                 Exception,
                 "Pre-review pipeline gate does not allow post-review job"):
-            item.freezeJobGraph()
+            item.freezeJobGraph(self.layout)
 
 
 class TestJobTimeData(BaseTestCase):
@@ -774,3 +772,38 @@ class TestFreezable(BaseTestCase):
         # still shows the ordered behavior.
         self.assertTrue(isinstance(o.odict2, types.MappingProxyType))
         self.assertEqual(list(o.odict2.keys()), seq)
+
+
+class TestRef(BaseTestCase):
+    def test_ref_equality(self):
+        change1 = model.Change('project1')
+        change1.ref = '/change1'
+        change1b = model.Change('project1')
+        change1b.ref = '/change1'
+        change2 = model.Change('project2')
+        change2.ref = '/change2'
+        self.assertFalse(change1.equals(change2))
+        self.assertTrue(change1.equals(change1b))
+
+        tag1 = model.Tag('project1')
+        tag1.ref = '/tag1'
+        tag1b = model.Tag('project1')
+        tag1b.ref = '/tag1'
+        tag2 = model.Tag('project2')
+        tag2.ref = '/tag2'
+        self.assertFalse(tag1.equals(tag2))
+        self.assertTrue(tag1.equals(tag1b))
+
+        self.assertFalse(tag1.equals(change1))
+
+        branch1 = model.Branch('project1')
+        branch1.ref = '/branch1'
+        branch1b = model.Branch('project1')
+        branch1b.ref = '/branch1'
+        branch2 = model.Branch('project2')
+        branch2.ref = '/branch2'
+        self.assertFalse(branch1.equals(branch2))
+        self.assertTrue(branch1.equals(branch1b))
+
+        self.assertFalse(branch1.equals(change1))
+        self.assertFalse(branch1.equals(tag1))
