@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -196,4 +196,9 @@ def main():
 
 
 if __name__ == '__main__':
+    print(
+        "This script is deprecated. Use `zuul-client encrypt` instead. "
+        "Please refer to https://zuul-ci.org/docs/zuul-client/ "
+        "for more details on how to use zuul-client."
+    )
     main()
