@@ -23,6 +23,7 @@ then
     pip install nodeenv
     # Initialize nodeenv and tell it to re-use the currently active virtualenv
     attempts=0
+    set +e
     until nodeenv --python-virtualenv -n 14.3.0 ; do
         ((attempts++))
         if [[ $attempts > 2 ]]
@@ -31,6 +32,7 @@ then
             exit 1
         fi
     done
+    set -e
     # Use -g because inside of the virtualenv '-g' means 'install into the'
     # virtualenv - as opposed to installing into the local node_modules.
     # Avoid writing a package-lock.json file since we don't use it.
@@ -39,9 +41,34 @@ then
 fi
 if [[ ! -f zuul/web/static/status.html ]]
 then
+    mkdir -p zuul/web/static
+    ln -sfn ../zuul/web/static web/build
     pushd web/
-        yarn install
+        if [[ -n "${YARN_REGISTRY}" ]]
+        then
+            echo "Using yarn registry: ${YARN_REGISTRY}"
+            sed -i "s#https://registry.yarnpkg.com#${YARN_REGISTRY}#" yarn.lock
+        fi
+
+        # Be forgiving of package retrieval errors
+        attempts=0
+        set +e
+        until yarn install --verbose; do
+            ((attempts++))
+            if [[ $attempts > 2 ]]
+            then
+                echo "Failed installing npm packages"
+                exit 1
+            fi
+        done
+        set -e
+
         yarn build
+        if [[ -n "${YARN_REGISTRY}" ]]
+        then
+            echo "Resetting yarn registry"
+            sed -i "s#${YARN_REGISTRY}#https://registry.yarnpkg.com#" yarn.lock
+        fi
     popd
 fi
 pip install $*
