@@ -120,7 +120,7 @@ class ChangePanel extends React.Component {
   }
 
   renderProgressBar (change) {
-    let jobPercent = Math.floor(100 / change.jobs.length)
+    let jobPercent = (100 / change.jobs.length).toFixed(2)
     return (
       <div className='progress zuul-change-total-result'>
         {change.jobs.map((job, idx) => {
@@ -203,7 +203,7 @@ class ChangePanel extends React.Component {
 
     return (
       <div className='progress zuul-job-result'
-            title={title}>
+        title={title}>
         <div className={'progress-bar ' + className}
           role='progressbar'
           aria-valuenow={progressPercent}
@@ -215,8 +215,8 @@ class ChangePanel extends React.Component {
     )
   }
 
-  renderJobStatusLabel (result) {
-    let className
+  renderJobStatusLabel (job, result) {
+    let className, title
     switch (result) {
       case 'success':
         className = 'label-success'
@@ -234,21 +234,34 @@ class ChangePanel extends React.Component {
       case 'skipped':
         className = 'label-info'
         break
-      // 'in progress' 'queued' 'lost' 'aborted' 'waiting' ...
+      case 'waiting':
+        className = 'label-default'
+        if (job.waiting_status !== null) {
+          title = 'Waiting on ' + job.waiting_status
+        }
+        break
+      // 'in progress' 'queued' 'lost' 'aborted' ...
       default:
         className = 'label-default'
     }
 
     return (
-      <span className={'zuul-job-result label ' + className}>{result}</span>
+      <span className={'zuul-job-result label ' + className} title={title}>{result}</span>
     )
   }
 
   renderJob (job) {
     const { tenant } = this.props
     let job_name = job.name
+    let ordinal_rules = new Intl.PluralRules('en', {type: 'ordinal'})
+    const suffixes = {
+      one: 'st',
+      two: 'nd',
+      few: 'rd',
+      other: 'th'
+    }
     if (job.tries > 1) {
-      job_name = job_name + ' (' + job.tries + '. attempt)'
+        job_name = job_name + ' (' + job.tries + suffixes[ordinal_rules.select(job.tries)] + ' attempt)'
     }
     let name = ''
     if (job.result !== null) {
@@ -272,7 +285,7 @@ class ChangePanel extends React.Component {
       resultBar = this.renderJobProgressBar(
         job.elapsed_time, job.remaining_time)
     } else {
-      resultBar = this.renderJobStatusLabel(result)
+      resultBar = this.renderJobStatusLabel(job, result)
     }
 
     return (
@@ -306,7 +319,7 @@ class ChangePanel extends React.Component {
     const header = (
       <div className='panel panel-default zuul-change'>
         <div className='panel-heading zuul-patchset-header'
-             onClick={this.onClick}>
+          onClick={this.onClick}>
           <div className='row'>
             <div className='col-xs-8'>
               <span className='change_project'>{change.project}</span>
