@@ -15,6 +15,7 @@
 import * as React from 'react'
 import PropTypes from 'prop-types'
 import { Badge } from 'patternfly-react'
+import { Tooltip } from '@patternfly/react-core'
 
 import ChangeQueue from './ChangeQueue'
 
@@ -95,7 +96,7 @@ class Pipeline extends React.Component {
       changes.forEach(change => {
         filters.forEach(changeFilter => {
           if (changeFilter && (
-              (change.project && change.project.indexOf(changeFilter) !== -1) ||
+            (change.project && change.project.indexOf(changeFilter) !== -1) ||
               (change.id && change.id.indexOf(changeFilter) !== -1))) {
             found = true
             return
@@ -116,26 +117,39 @@ class Pipeline extends React.Component {
     const { pipeline, filter, expanded } = this.props
     const count = this.createTree(pipeline)
     return (
-      <div className="zuul-pipeline col-sm-6 col-md-4">
+      <div className="pf-c-content zuul-pipeline col-sm-6 col-md-4">
         <div className="zuul-pipeline-header">
-          <h3>{pipeline.name} <Badge>{count}</Badge></h3>
+          <h3>
+            {pipeline.name} <Badge>{count}</Badge>
+          </h3>
           {pipeline.description ? (
             <small>
               <p>{pipeline.description.split(/\r?\n\r?\n/)}</p>
             </small>) : ''}
+          <Tooltip position="top"
+            content={<div>Unprocessed pipeline specific events</div>}>
+            <small>
+              <em>
+                Events:&nbsp;
+                {pipeline.trigger_events} trigger events,&nbsp;
+                {pipeline.management_events} management events,&nbsp;
+                {pipeline.result_events} results.
+              </em>
+            </small>
+          </Tooltip>
         </div>
         {pipeline.change_queues.filter(item => item.heads.length > 0)
-         .filter(item => (!filter || (
-           filter.indexOf(pipeline.name) !== -1 ||
+          .filter(item => (!filter || (
+            filter.indexOf(pipeline.name) !== -1 ||
              this.filterQueue(item, filter)
-         )))
+          )))
           .map(changeQueue => (
             <ChangeQueue
               queue={changeQueue}
               expanded={expanded}
               pipeline={pipeline.name}
               key={changeQueue.uuid}
-              />
+            />
           ))}
       </div>
     )
