@@ -700,11 +700,8 @@ class Client(zuul.cmd.ZuulApp):
             'number': {
                 'title': 'Number'
             },
-            'node_labels': {
-                'title': 'Node Labels'
-            },
-            'node_name': {
-                'title': 'Node Name'
+            'nodeset': {
+                'title': 'Nodeset'
             },
             'worker.name': {
                 'title': 'Worker'
@@ -718,8 +715,16 @@ class Client(zuul.cmd.ZuulApp):
         from zuul import scheduler
         from zuul import configloader
         self.configure_connections(source_only=True)
-        sched = scheduler.Scheduler(self.config, self.connections,
-                                    self, testonly=True)
+
+        class SchedulerConfig(scheduler.Scheduler):
+            # A custom scheduler constructor adapted for config check
+            # to avoid loading runtime clients.
+            def __init__(self, config, connections):
+                self.config = config
+                self.connections = connections
+                self.unparsed_config_cache = None
+
+        sched = SchedulerConfig(self.config, self.connections)
         loader = configloader.ConfigLoader(
             sched.connections, sched, None, None)
         tenant_config, script = sched._checkTenantSourceConf(self.config)
