@@ -16,7 +16,6 @@ import copy
 import re
 import time
 import urllib.parse
-
 import dateutil.parser
 
 from zuul.model import EventFilter, RefFilter
@@ -31,6 +30,8 @@ EMPTY_GIT_REF = '0' * 40  # git sha of all zeros, used during creates/deletes
 class GerritChange(Change):
     def __init__(self, project):
         super(GerritChange, self).__init__(project)
+        self.status = None
+        self.wip = None
         self.approvals = []
 
     def update(self, data, connection):
@@ -45,7 +46,7 @@ class GerritChange(Change):
         if 'project' not in data:
             raise exceptions.ChangeNotFound(self.number, self.patchset)
         self.project = connection.source.getProject(data['project'])
-        self.id = data['id']
+        self.commit_id = str(data['currentPatchSet']['revision'])
         self.branch = data['branch']
         self.url = data['url']
         urlparse = urllib.parse.urlparse(connection.baseurl)
@@ -73,12 +74,13 @@ class GerritChange(Change):
         else:
             self.is_current_patchset = False
         self.files = files
-
+        self.id = data['id']
         self.is_merged = data.get('status', '') == 'MERGED'
         self.approvals = data['currentPatchSet'].get('approvals', [])
         self.open = data['open']
         self.status = data['status']
-        self.owner = data['owner']
+        self.wip = data.get('wip', False)
+        self.owner = data['owner'].get('username')
         self.message = data['commitMessage']
 
         self.missing_labels = set()
@@ -99,6 +101,7 @@ class GerritChange(Change):
         if self.patchset is None:
             self.patchset = str(current_revision['_number'])
         self.project = connection.source.getProject(data['project'])
+        self.commit_id = str(data['current_revision'])
         self.id = data['change_id']
         self.branch = data['branch']
         self.url = '%s/%s' % (baseurl, self.number)
@@ -147,7 +150,8 @@ class GerritChange(Change):
             self.missing_labels.add(label_name)
         self.open = data['status'] == 'NEW'
         self.status = data['status']
-        self.owner = data['owner']
+        self.wip = data.get('work_in_progress', False)
+        self.owner = data['owner'].get('username')
         self.message = current_revision['commit']['message']
 
 
@@ -158,6 +162,22 @@ class GerritTriggerEvent(TriggerEvent):
         self.approvals = []
         self.uuid = None
         self.scheme = None
+        self.patchsetcomments = None
+
+    def toDict(self):
+        d = super().toDict()
+        d["approvals"] = self.approvals
+        d["uuid"] = self.uuid
+        d["scheme"] = self.scheme
+        d["patchsetcomments"] = self.patchsetcomments
+        return d
+
+    def updateFromDict(self, d):
+        super().updateFromDict(d)
+        self.approvals = d["approvals"]
+        self.uuid = d["uuid"]
+        self.scheme = d["scheme"]
+        self.patchsetcomments = d["patchsetcomments"]
 
     def __repr__(self):
         ret = '<GerritTriggerEvent %s %s' % (self.type,
@@ -275,12 +295,12 @@ class GerritApprovalFilter(object):
 
 
 class GerritEventFilter(EventFilter, GerritApprovalFilter):
-    def __init__(self, trigger, types=[], branches=[], refs=[],
-                 event_approvals={}, comments=[], emails=[], usernames=[],
-                 required_approvals=[], reject_approvals=[], uuid=None,
-                 scheme=None, ignore_deletes=True):
+    def __init__(self, connection_name, trigger, types=[], branches=[],
+                 refs=[], event_approvals={}, comments=[], emails=[],
+                 usernames=[], required_approvals=[], reject_approvals=[],
+                 uuid=None, scheme=None, ignore_deletes=True):
 
-        EventFilter.__init__(self, trigger)
+        EventFilter.__init__(self, connection_name, trigger)
 
         GerritApprovalFilter.__init__(self,
                                       required_approvals=required_approvals,
@@ -305,6 +325,7 @@ class GerritEventFilter(EventFilter, GerritApprovalFilter):
 
     def __repr__(self):
         ret = '<GerritEventFilter'
+        ret += ' connection: %s' % self.connection_name
 
         if self._types:
             ret += ' types: %s' % ', '.join(self._types)
@@ -338,6 +359,9 @@ class GerritEventFilter(EventFilter, GerritApprovalFilter):
         return ret
 
     def matches(self, event, change):
+        if not super().matches(event, change):
+            return False
+
         # event types are ORed
         matches_type = False
         for etype in self.types:
@@ -379,6 +403,11 @@ class GerritEventFilter(EventFilter, GerritApprovalFilter):
             if (event.comment is not None and
                 comment_re.search(event.comment)):
                 matches_comment_re = True
+            if event.patchsetcomments is not None:
+                for comment in event.patchsetcomments:
+                    if (comment is not None and
+                        comment_re.search(comment)):
+                        matches_comment_re = True
         if self.comments and not matches_comment_re:
             return False
 
