@@ -46,7 +46,9 @@ class GerritSource(BaseSource):
     def isMerged(self, change, head=None):
         return self.connection.isMerged(change, head)
 
-    def canMerge(self, change, allow_needs, event=None):
+    def canMerge(self, change, allow_needs, event=None, allow_refresh=False):
+        # Gerrit changes have no volatile data that cannot be updated via
+        # events and thus needs not to act on allow_refresh.
         return self.connection.canMerge(change, allow_needs, event=event)
 
     def postConfig(self):
@@ -55,7 +57,7 @@ class GerritSource(BaseSource):
     def getChange(self, event, refresh=False):
         return self.connection.getChange(event, refresh)
 
-    def getChangeByURL(self, url):
+    def getChangeByURL(self, url, event):
         try:
             parsed = urllib.parse.urlparse(url)
         except ValueError:
@@ -71,11 +73,12 @@ class GerritSource(BaseSource):
         except ValueError:
             return None
         query = "change:%s" % (change_no,)
-        results = self.connection.simpleQuery(query)
+        results = self.connection.simpleQuery(query, event=event)
         if not results:
             return None
         change = self.connection._getChange(
-            results[0].number, results[0].current_patchset)
+            results[0].number, results[0].current_patchset,
+            event=event)
         return change
 
     def getChangesDependingOn(self, change, projects, tenant):
@@ -148,7 +151,7 @@ class GerritSource(BaseSource):
         return [f]
 
     def getRefForChange(self, change):
-        partial = change[-2:]
+        partial = str(change).zfill(2)[-2:]
         return "refs/changes/%s/%s/.*" % (partial, change)
 
 
