@@ -35,6 +35,7 @@ class GitConnection(BaseConnection):
         if 'baseurl' not in self.connection_config:
             raise Exception('baseurl is required for git connections in '
                             '%s' % self.connection_name)
+        self.watcher_thread = None
         self.baseurl = self.connection_config.get('baseurl')
         self.poll_timeout = float(
             self.connection_config.get('poll_delay', 3600 * 2))
@@ -136,6 +137,7 @@ class GitConnection(BaseConnection):
 
     def watcherCallback(self, data):
         event = GitTriggerEvent()
+        event.connection_name = self.connection_name
         event.type = 'ref-updated'
         event.timestamp = time.time()
         event.project_hostname = self.canonical_hostname
@@ -150,7 +152,7 @@ class GitConnection(BaseConnection):
         self.getChange(event)
         self.logEvent(event)
         # Pass the event to the scheduler
-        self.sched.addEvent(event)
+        self.sched.addTriggerEvent(self.driver_name, event)
 
     def onLoad(self):
         self.log.debug("Starting Git Watcher")
