@@ -22,6 +22,7 @@ import threading
 import time
 import json
 from collections import OrderedDict, defaultdict
+from collections.abc import Mapping
 from itertools import chain
 from json.decoder import JSONDecodeError
 from typing import List, Optional
@@ -752,7 +753,7 @@ class GithubEventConnector:
                         continue
                     self.connection.logEvent(event)
                     if isinstance(event, DequeueEvent):
-                        self.connection.sched.addManagementEvent(event)
+                        self.connection.sched.addChangeManagementEvent(event)
                     else:
                         self.connection.sched.addTriggerEvent(
                             self.connection.driver_name, event
@@ -773,7 +774,7 @@ class GithubEventConnector:
         return EventTuple(time.time(), body, event_type, delivery)
 
 
-class GithubUser(collections.Mapping):
+class GithubUser(Mapping):
     log = logging.getLogger('zuul.GithubUser')
 
     def __init__(self, username, connection, project_name):
