@@ -34,9 +34,27 @@ class PullRequest(Change):
         self.pr = None
         self.updated_at = None
         self.title = None
+        self.body_text = None
         self.reviews = []
         self.files = []
         self.labels = []
+        self.draft = None
+        self.review_decision = None
+        self.required_contexts = set()
+        self.contexts = set()
+        self.branch_protected = False
+
+    @property
+    def status(self):
+        return ["{}:{}:{}".format(*c) for c in self.contexts]
+
+    @property
+    def successful_contexts(self) -> set:
+        if not self.contexts:
+            return set()
+        return set(
+            s[1] for s in self.contexts if s[2] == 'success'
+        )
 
     def isUpdateOf(self, other):
         if (self.project == other.project and
@@ -56,7 +74,32 @@ class GithubTriggerEvent(TriggerEvent):
         self.unlabel = None
         self.action = None
         self.delivery = None
-        self.check_runs = None
+        self.check_run = None
+        self.status = None
+        self.commits = []
+
+    def toDict(self):
+        d = super().toDict()
+        d["title"] = self.title
+        d["label"] = self.label
+        d["unlabel"] = self.unlabel
+        d["action"] = self.action
+        d["delivery"] = self.delivery
+        d["check_run"] = self.check_run
+        d["status"] = self.status
+        d["commits"] = self.commits
+        return d
+
+    def updateFromDict(self, d):
+        super().updateFromDict(d)
+        self.title = d["title"]
+        self.label = d["label"]
+        self.unlabel = d["unlabel"]
+        self.action = d["action"]
+        self.delivery = d["delivery"]
+        self.check_run = d["check_run"]
+        self.status = d["status"]
+        self.commits = d["commits"]
 
     def isPatchsetCreated(self):
         if self.type == 'pull_request':
@@ -77,8 +120,8 @@ class GithubTriggerEvent(TriggerEvent):
             r.append('%s,%s' % (self.change_number, self.patch_number))
         if self.delivery:
             r.append('delivery: %s' % self.delivery)
-        if self.check_runs:
-            r.append('check_runs: %s' % self.check_runs)
+        if self.check_run:
+            r.append('check_run: %s' % self.check_run)
         return ' '.join(r)
 
 
@@ -217,12 +260,12 @@ class GithubCommonFilter(object):
 
 
 class GithubEventFilter(EventFilter, GithubCommonFilter):
-    def __init__(self, trigger, types=[], branches=[], refs=[],
-                 comments=[], actions=[], labels=[], unlabels=[],
+    def __init__(self, connection_name, trigger, types=[], branches=[],
+                 refs=[], comments=[], actions=[], labels=[], unlabels=[],
                  states=[], statuses=[], required_statuses=[],
                  check_runs=[], ignore_deletes=True):
 
-        EventFilter.__init__(self, trigger)
+        EventFilter.__init__(self, connection_name, trigger)
 
         GithubCommonFilter.__init__(self, required_statuses=required_statuses)
 
@@ -245,7 +288,7 @@ class GithubEventFilter(EventFilter, GithubCommonFilter):
 
     def __repr__(self):
         ret = '<GithubEventFilter'
-
+        ret += ' connection: %s' % self.connection_name
         if self._types:
             ret += ' types: %s' % ', '.join(self._types)
         if self._branches:
@@ -275,6 +318,9 @@ class GithubEventFilter(EventFilter, GithubCommonFilter):
         return ret
 
     def matches(self, event, change):
+        if not super().matches(event, change):
+            return False
+
         # event types are ORed
         matches_type = False
         for etype in self.types:
