@@ -19,6 +19,7 @@ import cherrypy
 import voluptuous as v
 import time
 import uuid
+import re
 import requests
 
 import dateutil.parser
@@ -396,6 +397,8 @@ class GitlabConnection(CachedBranchConnection):
             'canonical_hostname', self.server)
         self.webhook_token = self.connection_config.get(
             'webhook_token', '')
+        self.api_token_name = self.connection_config.get(
+            'api_token_name', '')
         self.api_token = self.connection_config.get(
             'api_token', '')
         self.gl_client = GitlabAPIClient(self.baseurl, self.api_token)
@@ -458,7 +461,19 @@ class GitlabConnection(CachedBranchConnection):
         return '%s/%s/merge_requests/%s' % (self.baseurl, project, number)
 
     def getGitUrl(self, project):
-        return '%s/%s.git' % (self.cloneurl, project.name)
+        cloneurl = '%s/%s.git' % (self.cloneurl, project.name)
+        # https://gitlab.com/gitlab-org/gitlab/-/issues/212953
+        # any login name can be used, but it's likely going to be reduce to
+        # username/token-name
+        if (cloneurl.startswith('http') and self.api_token_name != '' and
+            not re.match("http?://.+:.+@.+", cloneurl)):
+            cloneurl = '%s://%s:%s@%s/%s.git' % (
+                self.cloneurl.split('://')[0],
+                self.api_token_name,
+                self.api_token,
+                self.cloneurl.split('://')[1],
+                project.name)
+        return cloneurl
 
     def getChange(self, event, refresh=False):
         project = self.source.getProject(event.project_name)
