@@ -60,6 +60,7 @@ class PipelineManager(metaclass=ABCMeta):
         self.ref_filters = []
         # Cached dynamic layouts (layout uuid -> layout)
         self._layout_cache = {}
+        self.sql = self.sched.sql
 
     def __str__(self):
         return "<%s %s>" % (self.__class__.__name__, self.pipeline.name)
@@ -193,6 +194,10 @@ class PipelineManager(metaclass=ABCMeta):
                 self.log.error(
                     "Reporting item dequeue %s received: %s", item, ret
                 )
+        # This might be called after canceljobs, which also sets a
+        # non-final 'cancel' result.
+        self.sql.reportBuildsetEnd(item.current_build_set, 'dequeue',
+                                   final=False)
 
     def sendReport(self, action_reporters, item, message=None):
         """Sends the built message off to configured reporters.
@@ -415,6 +420,8 @@ class PipelineManager(metaclass=ABCMeta):
                         ci
                     ):
                         self.sendReport(actions, ci)
+                    self.sql.reportBuildsetEnd(ci.current_build_set,
+                                               'failure', final=True)
 
                     return False
 
@@ -547,6 +554,7 @@ class PipelineManager(metaclass=ABCMeta):
         # Without this check, all items with a valid result would be reported
         # twice.
         if not item.current_build_set.result and item.live:
+            item.setReportedResult('DEQUEUED')
             self.reportDequeue(item)
 
     def removeItem(self, item):
@@ -622,17 +630,47 @@ class PipelineManager(metaclass=ABCMeta):
         build_set = item.current_build_set
         log.debug("Requesting nodes for change %s", item.change)
         if self.sched.use_relative_priority:
-            priority = item.getNodePriority()
+            relative_priority = item.getNodePriority()
         else:
-            priority = 0
+            relative_priority = 0
         for job in jobs:
+            provider = self._getPausedParentProvider(build_set, job)
+            priority = self._calculateNodeRequestPriority(build_set, job)
+            tenant_name = build_set.item.pipeline.tenant.name
+            pipeline_name = build_set.item.pipeline.name
             req = self.sched.nodepool.requestNodes(
-                build_set, job, priority, event=item.event)
+                build_set.uuid, job, tenant_name, pipeline_name, provider,
+                priority, relative_priority, event=item.event)
             log.debug("Adding node request %s for job %s to item %s",
                       req, job, item)
             build_set.setJobNodeRequest(job.name, req)
         return True
 
+    def _getPausedParent(self, build_set, job):
+        job_graph = build_set.item.job_graph
+        if job_graph:
+            for parent in job_graph.getParentJobsRecursively(job.name):
+                build = build_set.getBuild(parent.name)
+                if build.paused:
+                    return build
+        return None
+
+    def _getPausedParentProvider(self, build_set, job):
+        build = self._getPausedParent(build_set, job)
+        if build:
+            nodeset = build_set.getJobNodeSet(build.job.name)
+            if nodeset and nodeset.nodes:
+                return list(nodeset.nodes.values())[0].provider
+        return None
+
+    def _calculateNodeRequestPriority(self, build_set, job):
+        precedence_adjustment = 0
+        precedence = build_set.item.pipeline.precedence
+        if self._getPausedParent(build_set, job):
+            precedence_adjustment = -1
+        initial_precedence = model.PRIORITY_MAP[precedence]
+        return max(0, initial_precedence + precedence_adjustment)
+
     def _executeJobs(self, item, jobs):
         log = get_annotated_logger(self.log, item.event)
         log.debug("Executing jobs for change %s", item.change)
@@ -640,10 +678,6 @@ class PipelineManager(metaclass=ABCMeta):
         for job in jobs:
             log.debug("Found job %s for change %s", job, item.change)
             try:
-                nodeset = item.current_build_set.getJobNodeSet(job.name)
-                self.sched.nodepool.useNodeSet(
-                    nodeset, build_set=item.current_build_set,
-                    event=item.event)
                 self.sched.executor.execute(
                     job, item, self.pipeline,
                     build_set.dependent_changes,
@@ -683,6 +717,11 @@ class PipelineManager(metaclass=ABCMeta):
         # reported immediately afterwards during queue processing.
         if (prime and item.current_build_set.ref and not
                 item.didBundleStartReporting()):
+            # Force a dequeued result here because we haven't actually
+            # reported the item, but we are done with this buildset.
+            self.sql.reportBuildsetEnd(
+                item.current_build_set, 'dequeue', final=False,
+                result='DEQUEUED')
             item.resetAllBuilds()
 
         for job in jobs_to_cancel:
@@ -845,14 +884,14 @@ class PipelineManager(metaclass=ABCMeta):
         return self.getLayout(parent_item)
 
     def getLayout(self, item):
+        log = get_annotated_logger(self.log, item.event)
         layout = self._layout_cache.get(item.layout_uuid)
         if layout:
-            self.log.debug("Using cached layout %s for item %s",
-                           layout.uuid, item)
+            log.debug("Using cached layout %s for item %s", layout.uuid, item)
             return layout
 
         if item.layout_uuid:
-            self.log.debug("Re-calculating layout for item %s", item)
+            log.debug("Re-calculating layout for item %s", item)
 
         layout = self._getLayout(item)
         if layout:
@@ -861,6 +900,7 @@ class PipelineManager(metaclass=ABCMeta):
         return layout
 
     def _getLayout(self, item):
+        log = get_annotated_logger(self.log, item.event)
         if item.item_ahead:
             if (
                 (item.item_ahead.live and not item.item_ahead.job_graph) or
@@ -892,7 +932,7 @@ class PipelineManager(metaclass=ABCMeta):
         if build_set.unable_to_merge:
             return self.getFallbackLayout(item)
 
-        self.log.debug("Preparing dynamic layout for: %s" % item.change)
+        log.debug("Preparing dynamic layout for: %s" % item.change)
         return self._loadDynamicLayout(item)
 
     def _branchesForRepoState(self, projects, tenant, items=None):
@@ -1114,6 +1154,8 @@ class PipelineManager(metaclass=ABCMeta):
                 item.setConfigError("Unable to freeze job graph: %s" %
                                     (str(e)))
                 return False
+            if item.job_graph and len(item.job_graph.jobs) > 0:
+                self.sql.reportBuildsetStart(build_set)
 
         # At this point we know all frozen jobs and their repos so update the
         # repo state with all missing repos.
@@ -1257,6 +1299,10 @@ class PipelineManager(metaclass=ABCMeta):
         if item.live and not dequeued and self.sched.use_relative_priority:
             priority = item.getNodePriority()
             for node_request in item.current_build_set.node_requests.values():
+                if node_request.fulfilled:
+                    # If the node request is already fulfilled, there is no
+                    # need to update the relative priority.
+                    continue
                 if node_request.relative_priority != priority:
                     self.sched.nodepool.reviseRequest(
                         node_request, priority)
@@ -1274,6 +1320,8 @@ class PipelineManager(metaclass=ABCMeta):
         for ri in reported_items:
             ri.setReportedResult('FAILURE')
             self.sendReport(actions, ri)
+            self.sql.reportBuildsetEnd(ri.current_build_set,
+                                       'failure', final=True)
 
     def processQueue(self):
         # Do whatever needs to be done for each change in the queue
@@ -1315,6 +1363,7 @@ class PipelineManager(metaclass=ABCMeta):
     def onBuildStarted(self, build):
         log = get_annotated_logger(self.log, build.zuul_event_id)
         log.debug("Build %s started", build)
+        self.sql.reportBuildStart(build)
         return True
 
     def onBuildPaused(self, build):
@@ -1406,21 +1455,18 @@ class PipelineManager(metaclass=ABCMeta):
 
         return True
 
-    def onFilesChangesCompleted(self, event):
-        build_set = event.build_set
+    def onFilesChangesCompleted(self, event, build_set):
         item = build_set.item
         item.change.files = event.files
         build_set.files_state = build_set.COMPLETE
 
-    def onMergeCompleted(self, event):
-        build_set = event.build_set
+    def onMergeCompleted(self, event, build_set):
         if build_set.merge_state == build_set.COMPLETE:
-            self._onGlobalRepoStateCompleted(event)
+            self._onGlobalRepoStateCompleted(event, build_set)
         else:
-            self._onMergeCompleted(event)
+            self._onMergeCompleted(event, build_set)
 
-    def _onMergeCompleted(self, event):
-        build_set = event.build_set
+    def _onMergeCompleted(self, event, build_set):
         item = build_set.item
         item.change.containing_branches = event.item_in_branches
         build_set.merge_state = build_set.COMPLETE
@@ -1439,40 +1485,40 @@ class PipelineManager(metaclass=ABCMeta):
             self.log.info("Unable to merge change %s" % item.change)
             item.setUnableToMerge()
 
-    def _onGlobalRepoStateCompleted(self, event):
+    def _onGlobalRepoStateCompleted(self, event, build_set):
         if not event.updated:
-            item = event.build_set.item
+            item = build_set.item
             self.log.info("Unable to get global repo state for change %s"
                           % item.change)
             item.setUnableToMerge()
         else:
-            repo_state = event.build_set.repo_state
+            repo_state = build_set.repo_state
             for connection in event.repo_state.keys():
                 if connection in repo_state:
                     repo_state[connection].update(event.repo_state[connection])
                 else:
                     repo_state[connection] = event.repo_state[connection]
-            event.build_set.repo_state_state = event.build_set.COMPLETE
+            build_set.repo_state_state = build_set.COMPLETE
 
-    def onNodesProvisioned(self, event):
+    def onNodesProvisioned(self, request, build_set):
         # TODOv3(jeblair): handle provisioning failure here
-        request = event.request
         log = get_annotated_logger(self.log, request.event_id)
 
-        build_set = request.build_set
-        build_set.jobNodeRequestComplete(request.job.name,
-                                         request.nodeset)
+        build_set.jobNodeRequestComplete(request.job_name, request.nodeset)
+        # TODO (felix): Check if the failed is still needed as the
+        # NodesProvisionedEvents are now in ZooKeeper.
         if request.failed or not request.fulfilled:
             log.info("Node request %s: failure for %s",
-                     request, request.job.name)
-            build_set.item.setNodeRequestFailure(request.job)
-            self._resumeBuilds(request.build_set)
+                     request, request.job_name)
+            job = build_set.item.getJob(request.job_name)
+            build_set.item.setNodeRequestFailure(job)
+            self._resumeBuilds(build_set)
             tenant = build_set.item.pipeline.tenant
-            tenant.semaphore_handler.release(build_set.item, request.job)
+            tenant.semaphore_handler.release(build_set.item, job)
 
         log.info("Completed node request %s for job %s of item %s "
                  "with nodes %s",
-                 request, request.job, build_set.item, request.nodeset)
+                 request, request.job_name, build_set.item, request.nodeset)
 
     def reportItem(self, item):
         log = get_annotated_logger(self.log, item.event)
@@ -1543,42 +1589,51 @@ class PipelineManager(metaclass=ABCMeta):
             log.debug("Project %s not in pipeline %s for change %s",
                       item.change.project, self.pipeline, item.change)
             project_in_pipeline = False
+            action = 'no-jobs'
             actions = self.pipeline.no_jobs_actions
             item.setReportedResult('NO_JOBS')
         elif item.getConfigErrors():
             log.debug("Invalid config for change %s", item.change)
             # TODOv3(jeblair): consider a new reporter action for this
+            action = 'merge-failure'
             actions = self.pipeline.merge_failure_actions
             item.setReportedResult('CONFIG_ERROR')
         elif item.didMergerFail():
             log.debug("Merger failure")
+            action = 'merge-failure'
             actions = self.pipeline.merge_failure_actions
             item.setReportedResult('MERGER_FAILURE')
         elif item.wasDequeuedNeedingChange():
             log.debug("Dequeued needing change")
+            action = 'failure'
             actions = self.pipeline.failure_actions
             item.setReportedResult('FAILURE')
         elif not item.getJobs():
             # We don't send empty reports with +1
             log.debug("No jobs for change %s", item.change)
+            action = 'no-jobs'
             actions = self.pipeline.no_jobs_actions
             item.setReportedResult('NO_JOBS')
         elif item.cannotMergeBundle():
             log.debug("Bundle can not be merged")
+            action = 'failure'
             actions = self.pipeline.failure_actions
             item.setReportedResult("FAILURE")
         elif item.isBundleFailing():
             log.debug("Bundle is failing")
+            action = 'failure'
             actions = self.pipeline.failure_actions
             item.setReportedResult("FAILURE")
             if not item.didAllJobsSucceed():
                 self.pipeline._consecutive_failures += 1
         elif item.didAllJobsSucceed() and not item.isBundleFailing():
             log.debug("success %s", self.pipeline.success_actions)
+            action = 'success'
             actions = self.pipeline.success_actions
             item.setReportedResult('SUCCESS')
             self.pipeline._consecutive_failures = 0
         else:
+            action = 'failure'
             actions = self.pipeline.failure_actions
             item.setReportedResult('FAILURE')
             self.pipeline._consecutive_failures += 1
@@ -1595,6 +1650,7 @@ class PipelineManager(metaclass=ABCMeta):
             ret = self.sendReport(actions, item)
             if ret:
                 log.error("Reporting item %s received: %s", item, ret)
+        self.sql.reportBuildsetEnd(item.current_build_set, action, final=True)
         return ret
 
     def reportStats(self, item, added=False):
