@@ -111,17 +111,20 @@ class Nodepool(object):
             self.statsd.incr(
                 key, value * duration, project=project, resource=resource)
 
-    def requestNodes(self, build_set, job, relative_priority, event=None):
+    def requestNodes(self, build_set_uuid, job, tenant_name, pipeline_name,
+                     provider, priority, relative_priority, event=None):
         log = get_annotated_logger(self.log, event)
         # Create a copy of the nodeset to represent the actual nodes
         # returned by nodepool.
         nodeset = job.nodeset.copy()
-        req = model.NodeRequest(self.hostname, build_set, job,
-                                nodeset, relative_priority, event=event)
+        req = model.NodeRequest(self.hostname, build_set_uuid, tenant_name,
+                                pipeline_name, job.name, nodeset, provider,
+                                relative_priority, event=event)
         self.requests[req.uid] = req
 
         if nodeset.nodes:
-            self.zk_nodepool.submitNodeRequest(req, self._updateNodeRequest)
+            self.zk_nodepool.submitNodeRequest(
+                req, priority, self._updateNodeRequest)
             # Logged after submission so that we have the request id
             log.info("Submitted node request %s", req)
             self.emitStats(req)
@@ -138,9 +141,9 @@ class Nodepool(object):
     def cancelRequest(self, request):
         log = get_annotated_logger(self.log, request.event_id)
         log.info("Canceling node request %s", request)
-        if request.uid in self.requests:
-            request.canceled = True
+        if not request.canceled:
             try:
+                request.canceled = True
                 self.zk_nodepool.deleteNodeRequest(request)
             except Exception:
                 log.exception("Error deleting node request:")
@@ -178,7 +181,9 @@ class Nodepool(object):
             except Exception:
                 log.exception("Unable to unlock node request %s", request)
 
-    def holdNodeSet(self, nodeset, request, build):
+    # TODO (felix): Switch back to use a build object here rather than the
+    # ansible_job once it's available via ZK.
+    def holdNodeSet(self, nodeset, request, ansible_job):
         '''
         Perform a hold on the given set of nodes.
 
@@ -189,21 +194,17 @@ class Nodepool(object):
         resources = defaultdict(int)
         nodes = nodeset.getNodes()
 
-        duration = None
-        project = None
-        tenant = None
-        if build:
-            project = build.build_set.item.change.project
-            tenant = build.build_set.item.pipeline.tenant.name
-        if (build and build.start_time and build.end_time and
-            build.build_set and build.build_set.item and
-            build.build_set.item.change and
-            build.build_set.item.change.project):
-            duration = build.end_time - build.start_time
-            self.log.info(
-                "Nodeset %s with %s nodes was in use "
-                "for %s seconds for build %s for project %s",
-                nodeset, len(nodeset.nodes), duration, build, project)
+        args = ansible_job.arguments
+        project = args["zuul"]["project"]["canonical_name"]
+        tenant = args["zuul"]["tenant"]
+        duration = 0
+        if ansible_job.end_time and ansible_job.time_starting_build:
+            duration = ansible_job.end_time - ansible_job.time_starting_build
+        self.log.info(
+            "Nodeset %s with %s nodes was in use for %s seconds for build %s "
+            "for project %s",
+            nodeset, len(nodeset.nodes), duration, ansible_job, project)
+
         for node in nodes:
             if node.lock is None:
                 raise Exception("Node %s is not locked" % (node,))
@@ -220,7 +221,7 @@ class Nodepool(object):
             self.zk_nodepool.storeNode(node)
 
         request.nodes.append(dict(
-            build=build.uuid,
+            build=ansible_job.build_request.uuid,
             nodes=[node.id for node in nodes],
         ))
         request.current_count += 1
@@ -262,21 +263,24 @@ class Nodepool(object):
                 self.emitStatsResourceCounters(
                     tenant, project_name, resources, duration)
 
-    def useNodeSet(self, nodeset, build_set=None, event=None):
-        self.log.info("Setting nodeset %s in use" % (nodeset,))
+    # TODO (felix): Switch back to use a build object here rather than the
+    # ansible_job once it's available via ZK.
+    def useNodeSet(self, nodeset, ansible_job=None):
+        self.log.info("Setting nodeset %s in use", nodeset)
         resources = defaultdict(int)
         for node in nodeset.getNodes():
             if node.lock is None:
-                raise Exception("Node %s is not locked" % (node,))
+                raise Exception("Node %s is not locked", node)
             node.state = model.STATE_IN_USE
             self.zk_nodepool.storeNode(node)
             if node.resources:
                 add_resources(resources, node.resources)
-        if build_set and resources:
+        if ansible_job and resources:
+            args = ansible_job.arguments
             # we have a buildset and thus also tenant and project so we
             # can emit project specific resource usage stats
-            tenant_name = build_set.item.pipeline.tenant.name
-            project_name = build_set.item.change.project.canonical_name
+            tenant_name = args["zuul"]["tenant"]
+            project_name = args["zuul"]["project"]["canonical_name"]
 
             self.current_resources_by_tenant.setdefault(
                 tenant_name, defaultdict(int))
@@ -289,24 +293,13 @@ class Nodepool(object):
                           resources)
             self.emitStatsResources()
 
-    def returnNodeSet(self, nodeset, build=None, zuul_event_id=None):
+    # TODO (felix): Switch back to use a build object here rather than the
+    # ansible_job once it's available via ZK.
+    def returnNodeSet(self, nodeset, ansible_job=None, zuul_event_id=None):
         log = get_annotated_logger(self.log, zuul_event_id)
         log.info("Returning nodeset %s", nodeset)
         resources = defaultdict(int)
-        duration = None
-        project = None
-        tenant = None
-        if build:
-            project = build.build_set.item.change.project
-            tenant = build.build_set.item.pipeline.tenant.name
-        if (build and build.start_time and build.end_time and
-            build.build_set and build.build_set.item and
-            build.build_set.item.change and
-            build.build_set.item.change.project):
-            duration = build.end_time - build.start_time
-            log.info("Nodeset %s with %s nodes was in use "
-                     "for %s seconds for build %s for project %s",
-                     nodeset, len(nodeset.nodes), duration, build, project)
+
         for node in nodeset.getNodes():
             if node.lock is None:
                 log.error("Node %s is not locked", node)
@@ -322,21 +315,33 @@ class Nodepool(object):
                                   "while unlocking:", node)
         self._unlockNodes(nodeset.getNodes())
 
+        if not ansible_job:
+            return
+
+        args = ansible_job.arguments
+        project = args["zuul"]["project"]["canonical_name"]
+        tenant = args["zuul"]["tenant"]
+        duration = 0
+        if ansible_job.end_time and ansible_job.time_starting_build:
+            duration = ansible_job.end_time - ansible_job.time_starting_build
+        log.info("Nodeset %s with %s nodes was in use "
+                 "for %s seconds for build %s for project %s",
+                 nodeset, len(nodeset.nodes), duration, ansible_job, project)
+
         # When returning a nodeset we need to update the gauges if we have a
         # build. Further we calculate resource*duration and increment their
         # tenant or project specific counters. With that we have both the
         # current value and also counters to be able to perform accounting.
-        if tenant and project and resources:
-            project_name = project.canonical_name
+        if resources:
             subtract_resources(
                 self.current_resources_by_tenant[tenant], resources)
             subtract_resources(
-                self.current_resources_by_project[project_name], resources)
+                self.current_resources_by_project[project], resources)
             self.emitStatsResources()
 
             if duration:
                 self.emitStatsResourceCounters(
-                    tenant, project_name, resources, duration)
+                    tenant, project, resources, duration)
 
     def unlockNodeSet(self, nodeset):
         self._unlockNodes(nodeset.getNodes())
@@ -349,27 +354,31 @@ class Nodepool(object):
                 self.log.exception("Error unlocking node:")
 
     def lockNodeSet(self, nodeset, request_id):
-        self._lockNodes(nodeset.getNodes(), request_id)
-
-    def _lockNodes(self, nodes, request_id):
         # Try to lock all of the supplied nodes.  If any lock fails,
         # try to unlock any which have already been locked before
         # re-raising the error.
         locked_nodes = []
         try:
-            for node in nodes:
+            for node in nodeset.getNodes():
                 if node.allocated_to != request_id:
-                    raise Exception("Node %s allocated to %s, not %s" %
-                                    (node.id, node.allocated_to, request_id))
+                    raise Exception("Node %s allocated to %s, not %s",
+                                    node.id, node.allocated_to, request_id)
                 self.log.debug("Locking node %s" % (node,))
                 self.zk_nodepool.lockNode(node, timeout=30)
+                # Check the allocated_to again to ensure that nodepool didn't
+                # re-allocate the nodes to a different node request while we
+                # were locking them.
+                if node.allocated_to != request_id:
+                    raise Exception(
+                        "Node %s was reallocated during locking %s, not %s",
+                        node.id, node.allocated_to, request_id)
                 locked_nodes.append(node)
         except Exception:
             self.log.exception("Error locking nodes:")
             self._unlockNodes(locked_nodes)
             raise
 
-    def _updateNodeRequest(self, request, deleted):
+    def _updateNodeRequest(self, request, priority, deleted):
         log = get_annotated_logger(self.log, request.event_id)
         # Return False to indicate that we should stop watching the
         # node.
@@ -389,7 +398,7 @@ class Nodepool(object):
             log.debug("Resubmitting lost node request %s", request)
             request.id = None
             self.zk_nodepool.submitNodeRequest(
-                request, self._updateNodeRequest)
+                request, priority, self._updateNodeRequest)
             # Stop watching this request node
             return False
         elif request.state in (model.STATE_FULFILLED, model.STATE_FAILED):
@@ -409,14 +418,17 @@ class Nodepool(object):
 
         return True
 
-    def acceptNodes(self, request, request_id):
+    def checkNodeRequest(self, request, request_id):
+        """
+        Called by the scheduler when it wants to accept a node request for
+        potential use of its nodes. The nodes itself will be accepted and
+        locked by the executor when the corresponding job is started.
+
+        :returns: False if there is a problem with the request (canceled or
+            retrying), True if it is ready to be acted upon (success or
+            failure).
+        """
         log = get_annotated_logger(self.log, request.event_id)
-
-        # Called by the scheduler when it wants to accept and lock
-        # nodes for (potential) use.  Return False if there is a
-        # problem with the request (canceled or retrying), True if it
-        # is ready to be acted upon (success or failure).
-
         log.info("Accepting node request %s", request)
 
         if request_id != request.id:
@@ -430,7 +442,7 @@ class Nodepool(object):
             return False
 
         # If we didn't request nodes and the request is fulfilled then just
-        # return. We don't have to do anything in this case. Further don't even
+        # reutrn. We don't have to do anything in this case. Further don't even
         # ask ZK for the request as empty requests are not put into ZK.
         if not request.nodeset.nodes and request.fulfilled:
             return True
@@ -444,11 +456,14 @@ class Nodepool(object):
             if not self.zk_nodepool.nodeRequestExists(request):
                 log.info("Request %s no longer exists, resubmitting",
                          request.id)
+                # Look up the priority from the old request id before resetting
+                # it.
+                priority = request.id.partition("-")[0]
                 request.id = None
                 request.state = model.STATE_REQUESTED
                 self.requests[request.uid] = request
                 self.zk_nodepool.submitNodeRequest(
-                    request, self._updateNodeRequest)
+                    request, priority, self._updateNodeRequest)
                 return False
         except Exception:
             # If we cannot retrieve the node request from ZK we probably lost
@@ -460,18 +475,29 @@ class Nodepool(object):
             request.failed = True
             return True
 
+        return True
+
+    def acceptNodes(self, request):
         locked = False
         if request.fulfilled:
-            # If the request suceeded, try to lock the nodes.
+            # If the request succeeded, try to lock the nodes.
             try:
                 self.lockNodeSet(request.nodeset, request.id)
                 locked = True
             except Exception:
+                log = get_annotated_logger(self.log, request.event_id)
                 log.exception("Error locking nodes:")
                 request.failed = True
 
         # Regardless of whether locking (or even the request)
         # succeeded, delete the request.
+        self.deleteNodeRequest(request, locked)
+
+        if request.failed:
+            raise Exception("Accepting nodes failed")
+
+    def deleteNodeRequest(self, request, locked=False):
+        log = get_annotated_logger(self.log, request.event_id)
         log.debug("Deleting node request %s", request)
         try:
             self.zk_nodepool.deleteNodeRequest(request)
@@ -483,4 +509,3 @@ class Nodepool(object):
             # them.
             if locked:
                 self.unlockNodeSet(request.nodeset)
-        return True
