@@ -330,7 +330,7 @@ class RPCListener(RPCListenerBase):
 
     def handle_tenant_list(self, job):
         output = []
-        for tenant_name, tenant in self.sched.abide.tenants.items():
+        for tenant_name, tenant in sorted(self.sched.abide.tenants.items()):
             queue_size = 0
             for pipeline_name, pipeline in tenant.layout.pipelines.items():
                 for queue in pipeline.queues:
@@ -509,7 +509,7 @@ class RPCListener(RPCListenerBase):
         nodeset = job.nodeset
         job.setBase(tenant.layout)
         uuid = '0' * 32
-        params = zuul.executor.common.construct_gearman_params(
+        params = zuul.executor.common.construct_build_params(
             uuid, self.sched, nodeset,
             job, item, pipeline)
         gear_job.sendWorkComplete(json.dumps(params, cls=ZuulJSONEncoder))
