@@ -29,7 +29,7 @@ from kazoo.recipe.election import Election
 
 from zuul import model
 from zuul.lib.collections import DefaultKeyDict
-from zuul.zk import ZooKeeperSimpleBase
+from zuul.zk import ZooKeeperSimpleBase, sharding
 
 RESULT_EVENT_TYPE_MAP = {
     "BuildCompletedEvent": model.BuildCompletedEvent,
@@ -46,12 +46,30 @@ MANAGEMENT_EVENT_TYPE_MAP = {
     "EnqueueEvent": model.EnqueueEvent,
     "PromoteEvent": model.PromoteEvent,
     "ReconfigureEvent": model.ReconfigureEvent,
-    "SmartReconfigureEvent": model.SmartReconfigureEvent,
     "TenantReconfigureEvent": model.TenantReconfigureEvent,
 }
 
+# /zuul/events/tenant   TENANT_ROOT
+#   /{tenant}           TENANT_NAME_ROOT
+#     /management       TENANT_MANAGEMENT_ROOT
+#     /trigger          TENANT_TRIGGER_ROOT
+#     /pipelines        PIPELINE_ROOT
+#       /{pipeline}     PIPELINE_NAME_ROOT
+#         /management   PIPELINE_MANAGEMENT_ROOT
+#         /trigger      PIPELINE_TRIGGER_ROOT
+#         /result       PIPELINE_RESULT_ROOT
+
 TENANT_ROOT = "/zuul/events/tenant"
-SCHEDULER_GLOBAL_ROOT = "/zuul/events/scheduler-global"
+TENANT_NAME_ROOT = TENANT_ROOT + "/{tenant}"
+TENANT_MANAGEMENT_ROOT = TENANT_NAME_ROOT + "/management"
+TENANT_TRIGGER_ROOT = TENANT_NAME_ROOT + "/trigger"
+PIPELINE_ROOT = TENANT_NAME_ROOT + "/pipelines"
+PIPELINE_NAME_ROOT = PIPELINE_ROOT + "/{pipeline}"
+PIPELINE_MANAGEMENT_ROOT = PIPELINE_NAME_ROOT + "/management"
+PIPELINE_TRIGGER_ROOT = PIPELINE_NAME_ROOT + "/trigger"
+PIPELINE_RESULT_ROOT = PIPELINE_NAME_ROOT + "/result"
+
+EVENT_DATA_ROOT = "/zuul/events/data"
 CONNECTION_ROOT = "/zuul/events/connection"
 
 # This is the path to the serialized from of the event in ZK (along
@@ -71,26 +89,9 @@ class EventPrefix(enum.Enum):
     TRIGGER = "300"
 
 
-class GlobalEventWatcher(ZooKeeperSimpleBase):
+class EventWatcher(ZooKeeperSimpleBase):
 
-    log = logging.getLogger("zuul.zk.event_queues.EventQueueWatcher")
-
-    def __init__(self, client, callback):
-        super().__init__(client)
-        self.callback = callback
-        self.kazoo_client.ensure_path(SCHEDULER_GLOBAL_ROOT)
-        self.kazoo_client.ChildrenWatch(
-            SCHEDULER_GLOBAL_ROOT, self._eventWatch
-        )
-
-    def _eventWatch(self, event_list):
-        if event_list:
-            self.callback()
-
-
-class PipelineEventWatcher(ZooKeeperSimpleBase):
-
-    log = logging.getLogger("zuul.zk.event_queues.EventQueueWatcher")
+    log = logging.getLogger("zuul.zk.event_queues.EventWatcher")
 
     def __init__(self, client, callback):
         super().__init__(client)
@@ -100,31 +101,46 @@ class PipelineEventWatcher(ZooKeeperSimpleBase):
         self.kazoo_client.ensure_path(TENANT_ROOT)
         self.kazoo_client.ChildrenWatch(TENANT_ROOT, self._tenantWatch)
 
+    def _makePipelineWatcher(self, tenant_name):
+        def watch(children, event=None):
+            return self._pipelineWatch(tenant_name, children)
+        return watch
+
     def _tenantWatch(self, tenants):
         for tenant_name in tenants:
-            tenant_path = "/".join((TENANT_ROOT, tenant_name))
-
-            if tenant_path in self.watched_tenants:
+            if tenant_name in self.watched_tenants:
                 continue
 
+            for path in (TENANT_MANAGEMENT_ROOT,
+                         TENANT_TRIGGER_ROOT):
+                path = path.format(tenant=tenant_name)
+                self.kazoo_client.ensure_path(path)
+                self.kazoo_client.ChildrenWatch(
+                    path, self._eventWatch, send_event=True)
+
+            pipelines_path = PIPELINE_ROOT.format(tenant=tenant_name)
+            self.kazoo_client.ensure_path(pipelines_path)
             self.kazoo_client.ChildrenWatch(
-                tenant_path,
-                lambda p: self._pipelineWatch(tenant_name, p),
-            )
-            self.watched_tenants.add(tenant_path)
+                pipelines_path, self._makePipelineWatcher(tenant_name))
+
+            self.watched_tenants.add(tenant_name)
 
     def _pipelineWatch(self, tenant_name, pipelines):
         for pipeline_name in pipelines:
-            pipeline_path = "/".join((TENANT_ROOT, tenant_name, pipeline_name))
-            if pipeline_path in self.watched_pipelines:
+            key = (tenant_name, pipeline_name)
+            if key in self.watched_pipelines:
                 continue
 
-            self.kazoo_client.ChildrenWatch(
-                pipeline_path,
-                self._eventWatch,
-                send_event=True,
-            )
-            self.watched_pipelines.add(pipeline_path)
+            for path in (PIPELINE_MANAGEMENT_ROOT,
+                         PIPELINE_TRIGGER_ROOT,
+                         PIPELINE_RESULT_ROOT):
+                path = path.format(tenant=tenant_name,
+                                   pipeline=pipeline_name)
+
+                self.kazoo_client.ensure_path(path)
+                self.kazoo_client.ChildrenWatch(
+                    path, self._eventWatch, send_event=True)
+            self.watched_pipelines.add(key)
 
     def _eventWatch(self, event_list, event=None):
         if event is None:
@@ -137,80 +153,7 @@ class PipelineEventWatcher(ZooKeeperSimpleBase):
 
 
 class ZooKeeperEventQueue(ZooKeeperSimpleBase, Iterable):
-    """Abstract API for events via ZooKeeper"""
-
-    log = logging.getLogger("zuul.zk.event_queues.ZooKeeperEventQueue")
-
-    def __init__(self, client, event_root):
-        super().__init__(client)
-        self.event_root = event_root
-        self.kazoo_client.ensure_path(self.event_root)
-
-    def _listEvents(self):
-        return self.kazoo_client.get_children(self.event_root)
-
-    def __len__(self):
-        try:
-            return len(self._listEvents())
-        except NoNodeError:
-            return 0
-
-    def hasEvents(self):
-        return bool(len(self))
-
-    def ack(self, event):
-        # Event.ack_ref is an EventAckRef, previously attached to an
-        # event object when it was de-serialized.
-        if not event.ack_ref:
-            raise RuntimeError("Cannot ack event %s without reference", event)
-        try:
-            self.kazoo_client.delete(
-                event.ack_ref.path,
-                version=event.ack_ref.version,
-                recursive=True,
-            )
-        except NoNodeError:
-            self.log.warning("Event %s was already acknowledged", event)
-
-    @property
-    def _event_create_path(self):
-        return f"{self.event_root}/"
-
-    def _put(self, data):
-        return self.kazoo_client.create(
-            self._event_create_path,
-            json.dumps(data).encode("utf-8"),
-            sequence=True,
-            makepath=True,
-        )
-
-    def _iterEvents(self):
-        try:
-            # We need to sort this ourself, since Kazoo doesn't guarantee any
-            # ordering of the returned children.
-            events = sorted(self._listEvents())
-        except NoNodeError:
-            return
-
-        for event_id in events:
-            path = "/".join((self.event_root, event_id))
-            # TODO: implement sharding of large events
-            data, zstat = self.kazoo_client.get(path)
-            try:
-                event = json.loads(data)
-            except json.JSONDecodeError:
-                self.log.exception("Malformed event data in %s", path)
-                self._remove(path)
-                continue
-            yield event, EventAckRef(path, zstat.version), zstat
-
-    def _remove(self, path, version=UNKNOWN_ZVERSION):
-        with suppress(NoNodeError):
-            self.kazoo_client.delete(path, version=version, recursive=True)
-
-
-class SchedulerEventQueue(ZooKeeperEventQueue):
-    """Abstract API for tenant specific events via ZooKeeper
+    """Abstract API for events via ZooKeeper
 
     The lifecycle of a global (not pipeline-specific) event is:
 
@@ -237,22 +180,157 @@ class SchedulerEventQueue(ZooKeeperEventQueue):
 
     """
 
-    log = logging.getLogger("zuul.zk.event_queues.SchedulerEventQueue")
+    log = logging.getLogger("zuul.zk.event_queues.ZooKeeperEventQueue")
 
-    def __init__(self, client, event_root, event_prefix):
-        super().__init__(client, event_root)
-        self.event_prefix = event_prefix
+    def __init__(self, client, event_root):
+        super().__init__(client)
+        self.event_root = event_root
+        self.kazoo_client.ensure_path(self.event_root)
+        self.kazoo_client.ensure_path(EVENT_DATA_ROOT)
 
     def _listEvents(self):
-        return [
-            e
-            for e in self.kazoo_client.get_children(self.event_root)
-            if e.startswith(self.event_prefix.value)
-        ]
+        return self.kazoo_client.get_children(self.event_root)
 
-    @property
-    def _event_create_path(self) -> str:
-        return "{}/{}-".format(self.event_root, self.event_prefix.value)
+    def __len__(self):
+        try:
+            data, stat = self.kazoo_client.get(self.event_root)
+            return stat.children_count
+        except NoNodeError:
+            return 0
+
+    def hasEvents(self):
+        return bool(len(self))
+
+    def ack(self, event):
+        # Event.ack_ref is an EventAckRef, previously attached to an
+        # event object when it was de-serialized.
+        if not event.ack_ref:
+            raise RuntimeError("Cannot ack event %s without reference", event)
+        if not self._remove(event.ack_ref.path, event.ack_ref.version):
+            self.log.warning("Event %s was already acknowledged", event)
+
+    def _put(self, data):
+        # Within a transaction, we need something after the / in order
+        # to cause the sequence node to be created under the node
+        # before the /.  So we use "seq" for that.  This will produce
+        # paths like event_root/seq-0000.
+        event_path = f"{self.event_root}/seq"
+
+        # Event data can be large, so we want to shard it.  But events
+        # also need to be atomic (we don't want an event listener to
+        # start processing a half-stored event).  A natural solution
+        # is to use a ZK transaction to write the sharded data along
+        # with the event.  However, our events are sequence nodes in
+        # order to maintain ordering, and we can't use our sharding
+        # helper to write shards underneath a sequence node inside the
+        # transaction because we don't know the path of the sequence
+        # node within the transaction.  To resolve this, we store the
+        # event data in two places: the event itself and associated
+        # metadata are in the event queue as a single sequence node.
+        # The event data are stored in a separate tree under a uuid.
+        # The event metadata includes the UUID of the data.  We call
+        # the separated data "side channel data" to indicate it's
+        # stored outside the main event queue.
+        #
+        # To make the API simpler to work with, we assume "event_data"
+        # contains the bulk of the data.  We extract it here into the
+        # side channel data, then in _iterEvents we re-constitute it
+        # into the dictionary.
+
+        side_channel_data = None
+        encoded_data = json.dumps(data).encode("utf-8")
+        if (len(encoded_data) > sharding.NODE_BYTE_SIZE_LIMIT
+            and 'event_data' in data):
+            # Get a unique data node
+            data_id = str(uuid.uuid4())
+            data_root = f'{EVENT_DATA_ROOT}/{data_id}'
+            data_path = f'{data_root}/seq'
+            side_channel_data = json.dumps(data['event_data']).encode("utf-8")
+            data = data.copy()
+            del data['event_data']
+            data['event_data_path'] = data_root
+            encoded_data = json.dumps(data).encode("utf-8")
+
+            tr = self.kazoo_client.transaction()
+            tr.create(data_root)
+
+            with sharding.BufferedShardWriter(tr, data_path) as stream:
+                stream.write(side_channel_data)
+
+            tr.create(event_path, encoded_data, sequence=True)
+            resp = self.client.commitTransaction(tr)
+            return resp[-1]
+        else:
+            return self.kazoo_client.create(
+                event_path, encoded_data, sequence=True)
+
+    def _iterEvents(self):
+        try:
+            # We need to sort this ourself, since Kazoo doesn't guarantee any
+            # ordering of the returned children.
+            events = sorted(self._listEvents())
+        except NoNodeError:
+            return
+
+        for event_id in events:
+            path = "/".join((self.event_root, event_id))
+
+            # Load the event metadata
+            data, zstat = self.kazoo_client.get(path)
+            try:
+                event = json.loads(data)
+            except json.JSONDecodeError:
+                self.log.exception("Malformed event data in %s", path)
+                self._remove(path)
+                continue
+
+            # Load the event data (if present); if that fails, the
+            # event is corrupt; delete and continue.
+            side_channel_path = event.get('event_data_path')
+            if side_channel_path:
+                try:
+                    with sharding.BufferedShardReader(
+                            self.kazoo_client, side_channel_path) as stream:
+                        side_channel_data = stream.read()
+                except NoNodeError:
+                    self.log.exception("Side channel data for %s "
+                                       "not found at %s",
+                                       path, side_channel_path)
+                    self._remove(path)
+                    continue
+
+                try:
+                    event_data = json.loads(side_channel_data)
+                except json.JSONDecodeError:
+                    self.log.exception("Malformed side channel "
+                                       "event data in %s",
+                                       side_channel_path)
+                    self._remove(path)
+                    continue
+                event['event_data'] = event_data
+
+            yield event, EventAckRef(path, zstat.version), zstat
+
+    def _remove(self, path, version=UNKNOWN_ZVERSION):
+        try:
+            # Find the side channel path
+
+            side_channel_path = None
+            data, zstat = self.kazoo_client.get(path)
+            try:
+                event = json.loads(data)
+                side_channel_path = event.get('event_data_path')
+            except json.JSONDecodeError:
+                pass
+
+            if side_channel_path:
+                with suppress(NoNodeError):
+                    self.kazoo_client.delete(side_channel_path, recursive=True)
+
+            self.kazoo_client.delete(path, version=version, recursive=True)
+            return True
+        except NoNodeError:
+            return False
 
 
 class ManagementEventResultFuture(ZooKeeperSimpleBase):
@@ -302,7 +380,7 @@ class ManagementEventResultFuture(ZooKeeperSimpleBase):
         return True
 
 
-class ManagementEventQueue(SchedulerEventQueue):
+class ManagementEventQueue(ZooKeeperEventQueue):
     """Management events via ZooKeeper"""
 
     RESULTS_ROOT = "/zuul/results/management"
@@ -394,8 +472,10 @@ class PipelineManagementEventQueue(ManagementEventQueue):
     )
 
     def __init__(self, client, tenant_name, pipeline_name):
-        event_root = "/".join((TENANT_ROOT, tenant_name, pipeline_name))
-        super().__init__(client, event_root, EventPrefix.MANAGEMENT)
+        event_root = PIPELINE_MANAGEMENT_ROOT.format(
+            tenant=tenant_name,
+            pipeline=pipeline_name)
+        super().__init__(client, event_root)
 
     @classmethod
     def createRegistry(cls, client):
@@ -416,11 +496,13 @@ class PipelineManagementEventQueue(ManagementEventQueue):
         return DefaultKeyDict(lambda p: cls(client, tenant_name, p))
 
 
-class GlobalManagementEventQueue(ManagementEventQueue):
-    log = logging.getLogger("zuul.zk.event_queues.GlobalManagementEventQueue")
+class TenantManagementEventQueue(ManagementEventQueue):
+    log = logging.getLogger("zuul.zk.event_queues.TenantManagementEventQueue")
 
-    def __init__(self, client):
-        super().__init__(client, SCHEDULER_GLOBAL_ROOT, EventPrefix.MANAGEMENT)
+    def __init__(self, client, tenant_name):
+        event_root = TENANT_MANAGEMENT_ROOT.format(
+            tenant=tenant_name)
+        super().__init__(client, event_root)
 
     def ackWithoutResult(self, event):
         """
@@ -431,15 +513,30 @@ class GlobalManagementEventQueue(ManagementEventQueue):
             for merged_event in event.merged_events:
                 super(ManagementEventQueue, self).ack(merged_event)
 
+    @classmethod
+    def createRegistry(cls, client):
+        """Create a tenant queue registry
 
-class PipelineResultEventQueue(SchedulerEventQueue):
+        Returns a dictionary of: tenant_name -> EventQueue
+
+        Queues are dynamically created with the originally supplied ZK
+        client as they are accessed via the registry (so new tenants
+        show up automatically).
+
+        """
+        return DefaultKeyDict(lambda t: cls(client, t))
+
+
+class PipelineResultEventQueue(ZooKeeperEventQueue):
     """Result events via ZooKeeper"""
 
     log = logging.getLogger("zuul.zk.event_queues.PipelineResultEventQueue")
 
     def __init__(self, client, tenant_name, pipeline_name):
-        event_root = "/".join((TENANT_ROOT, tenant_name, pipeline_name))
-        super().__init__(client, event_root, EventPrefix.RESULT)
+        event_root = PIPELINE_RESULT_ROOT.format(
+            tenant=tenant_name,
+            pipeline=pipeline_name)
+        super().__init__(client, event_root)
 
     @classmethod
     def createRegistry(cls, client):
@@ -480,14 +577,14 @@ class PipelineResultEventQueue(SchedulerEventQueue):
             yield event
 
 
-class TriggerEventQueue(SchedulerEventQueue):
+class TriggerEventQueue(ZooKeeperEventQueue):
     """Trigger events via ZooKeeper"""
 
     log = logging.getLogger("zuul.zk.event_queues.TriggerEventQueue")
 
     def __init__(self, client, event_root, connections):
         self.connections = connections
-        super().__init__(client, event_root, EventPrefix.TRIGGER)
+        super().__init__(client, event_root)
 
     def put(self, driver_name, event):
         data = {
@@ -513,18 +610,35 @@ class TriggerEventQueue(SchedulerEventQueue):
             yield event
 
 
-class GlobalTriggerEventQueue(TriggerEventQueue):
-    log = logging.getLogger("zuul.zk.event_queues.GlobalTriggerEventQueue")
+class TenantTriggerEventQueue(TriggerEventQueue):
+    log = logging.getLogger("zuul.zk.event_queues.TenantTriggerEventQueue")
 
-    def __init__(self, client, connections):
-        super().__init__(client, SCHEDULER_GLOBAL_ROOT, connections)
+    def __init__(self, client, connections, tenant_name):
+        event_root = TENANT_TRIGGER_ROOT.format(
+            tenant=tenant_name)
+        super().__init__(client, event_root, connections)
+
+    @classmethod
+    def createRegistry(cls, client, connections):
+        """Create a tenant queue registry
+
+        Returns a dictionary of: tenant_name -> EventQueue
+
+        Queues are dynamically created with the originally supplied ZK
+        client as they are accessed via the registry (so new tenants
+        show up automatically).
+
+        """
+        return DefaultKeyDict(lambda t: cls(client, connections, t))
 
 
 class PipelineTriggerEventQueue(TriggerEventQueue):
     log = logging.getLogger("zuul.zk.event_queues.PipelineTriggerEventQueue")
 
     def __init__(self, client, tenant_name, pipeline_name, connections):
-        event_root = "/".join((TENANT_ROOT, tenant_name, pipeline_name))
+        event_root = PIPELINE_TRIGGER_ROOT.format(
+            tenant=tenant_name,
+            pipeline=pipeline_name)
         super().__init__(client, event_root, connections)
 
     @classmethod
@@ -577,7 +691,7 @@ class ConnectionEventQueue(ZooKeeperEventQueue):
     def put(self, data):
         self.log.debug("Submitting connection event to queue %s: %s",
                        self.event_root, data)
-        self._put(data)
+        self._put({'event_data': data})
 
     def __iter__(self):
         for data, ack_ref, _ in self._iterEvents():
@@ -585,7 +699,9 @@ class ConnectionEventQueue(ZooKeeperEventQueue):
                 self.log.warning("Malformed event found: %s", data)
                 self._remove(ack_ref.path)
                 continue
-            event = model.ConnectionEvent.fromDict(data)
+            # TODO: We can assume event_data exists after 4.6.1 is released
+            event = model.ConnectionEvent.fromDict(
+                data.get('event_data', data))
             event.ack_ref = ack_ref
             yield event
 
