@@ -21,7 +21,7 @@ from kazoo.recipe.cache import TreeEvent
 from kazoo.recipe.lock import Lock
 
 import zuul.model
-from zuul.model import HoldRequest
+from zuul.model import HoldRequest, Node, NodeRequest, NodeSet
 from zuul.zk import ZooKeeperClient, ZooKeeperBase
 from zuul.zk.exceptions import LockException
 
@@ -366,7 +366,7 @@ class ZooKeeperNodepool(ZooKeeperBase):
             self.log.exception(
                 "Exception in hold request cache update for event: %s", event)
 
-    def submitNodeRequest(self, node_request, watcher):
+    def submitNodeRequest(self, node_request, priority, watcher):
         """
         Submit a request for nodes to Nodepool.
 
@@ -385,7 +385,7 @@ class ZooKeeperNodepool(ZooKeeperBase):
         node_request.created_time = time.time()
         data = node_request.toDict()
 
-        path = '{}/{:0>3}-'.format(self.REQUEST_ROOT, node_request.priority)
+        path = '{}/{:0>3}-'.format(self.REQUEST_ROOT, priority)
         path = self.kazoo_client.create(path, json.dumps(data).encode('utf8'),
                                         makepath=True, sequence=True,
                                         ephemeral=True)
@@ -396,10 +396,58 @@ class ZooKeeperNodepool(ZooKeeperBase):
             if value:
                 self.updateNodeRequest(node_request, value)
             deleted = (value is None)  # data *are* none
-            return watcher(node_request, deleted)
+            return watcher(node_request, priority, deleted)
 
         self.kazoo_client.DataWatch(path, callback)
 
+    def getNodeRequest(self, node_request_id, nodeset=None):
+        """
+        Retrieve a NodeRequest from a given path in ZooKeeper.
+
+        The serialized version of the NodeRequest doesn't contain a NodeSet, so
+        we have to add this to the request manually. The nodeset provided to
+        this method will be set on the NodeRequest before updating it. This
+        will ensure that all nodes are updated as well.
+        """
+
+        path = f"{self.REQUEST_ROOT}/{node_request_id}"
+        try:
+            data, stat = self.kazoo_client.get(path)
+        except NoNodeError:
+            return None
+
+        if not data:
+            return None
+
+        json_data = json.loads(data.decode("utf-8"))
+
+        obj = NodeRequest.fromDict(json_data)
+        if nodeset is None:
+            # If no NodeSet is provided, we create one on-the-fly based on the
+            # list of labels (node_types) stored in the NodeRequest's znode
+            # data.
+            # This is necessary as the logic in the updateNodeRequest() method
+            # below will update each node "in-place" an thus, we have to ensure
+            # that the NodeRequest has a valid NodeSet with all nodes available
+            # in advance.
+            # This is only used to return the nodes to nodepool in case
+            # something went wrong and the original NodeRequest and/or NodeSet
+            # objects are not available anymore.
+            nodeset = NodeSet()
+            for i, node_type in enumerate(json_data["node_types"]):
+                node = Node(name=f"{node_type}-{i}", label=node_type)
+                nodeset.addNode(node)
+
+        obj.nodeset = nodeset
+        # Using updateNodeRequest() here will ensure that the nodes are also
+        # updated. Doing the update in here directly rather than calling it
+        # from the outside afterwards, saves us one call to ZooKeeper as the
+        # data we just retrieved can directly be reused.
+        self.updateNodeRequest(obj, data)
+        obj.id = node_request_id
+        obj.stat = stat
+        return obj
+
     def deleteNodeRequest(self, node_request):
         """
         Delete a request for nodes.
@@ -515,7 +563,7 @@ class ZooKeeperNodepool(ZooKeeperBase):
         """
 
         if node.lock is None:
-            raise LockException("Node %s does not hold a lock" % (node,))
+            raise LockException(f"Node {node} does not hold a lock")
         node.lock.release()
         node.lock = None
 
